# Quantile-Quantile plots for distributions (data) analysis

import fcodes_fast
import numpy as np
import argparse
from emda import iotools
from matplotlib import pyplot as plt

cmdl_parser = argparse.ArgumentParser(description='Prepare Quantile-Quantile plot\n')
cmdl_parser.add_argument('-f1', '--input_map1', required=True, help='Input filename for map1')
cmdl_parser.add_argument('-f2', '--input_map2', required=True, help='Input filename for map2')
cmdl_parser.add_argument('-v', '--verbose', default=False,
                         help='Verbose output')

def pass_cif_sf(cif_name):
    import gemmi
    import numpy as np
    doc = gemmi.cif.read(cif_name)
    rblock = gemmi.as_refln_blocks(doc)[0]
    #h = rblock.make_array_int('index_h', -1000)
    #k = rblock.make_array_int('index_k', -1000)
    #l = rblock.make_array_int('index_l', -1000)
    fobs = rblock.make_array_float('F_meas_au')
    resol = rblock.make_1_d2_array().round(4)
    resol = np.sqrt(1./resol)
    return fobs,resol

def make_xycols(set1,set2,resol1,resol2):
    import fcodes_fast
    lowres = np.max([np.max(resol1), np.max(resol2)])
    higres = np.min([np.min(resol1), np.min(resol2)])
    res_arr = fcodes_fast.setbin(10,lowres,higres)
    bin_idx1 = fcodes_fast.binarr_1d(res_arr,resol1,len(res_arr),len(resol1))
    bin_idx2 = fcodes_fast.binarr_1d(res_arr,resol2,len(res_arr),len(resol2))
    xq_lst = []
    yq_lst = []
    for ibin in range(len(res_arr)):
        print(ibin)
        x = set1 * (bin_idx1 == ibin)
        x = x[~np.isnan(x)] # remove NaN values
        #x = x[x > 0.]
        #if x.size > 0: x = x/np.max(x)
        # normalize to variance of Fobs
        x_variance = np.average((x - np.average(x))**2)
        if x.size > 0: x = x/np.sqrt(x_variance)
        y = set2 * (bin_idx2 == ibin)
        y = y[~np.isnan(y)]
        #y = y[y > 0.]
        #if y.size > 0: y = y/np.max(y)
        y_variance = np.average((y - np.average(y))**2)
        if y.size > 0: y = y/np.sqrt(y_variance)
        x_q, y_q = qqplot_simple(x, y)
        xq_lst.append(x_q)
        yq_lst.append(y_q)
        print(x_variance, y_variance)
    return xq_lst, yq_lst

def using_3d_data(uc, f1, f2):
    # create bin_idx
    nx,ny,nz = f1.shape
    maxbin = np.amax(np.array([nx//2,ny//2,nz//2]))
    nbin,_,bin_idx = fcodes_fast.resolution_grid(uc,1,maxbin,nx,ny,nz)
    #nbin = 49
    #res_arr,bin_idx = fcodes_fast.bin_idx_with_given_nbin(uc,nbin,nx,ny,nz)
    I1 = np.real(f1 * f1.conj())
    I2 = np.real(f2 * f2.conj())
    xq_lst = []
    yq_lst = []
    for ibin in range(15,20):
        x = (I1[bin_idx == ibin]).flatten()
        x = x[~np.isnan(x)] # remove NaN values
        x_variance = np.average((x - np.average(x))**2)
        # normalise to sqrt of variance
        if x.size > 0: x = x/np.sqrt(x_variance)
        y = (I2[bin_idx == ibin]).flatten()
        y = y[~np.isnan(y)]
        y_variance = np.average((y - np.average(y))**2)
        if y.size > 0: y = y/np.sqrt(y_variance)
        xq_lst.append(x)
        yq_lst.append(y)
        print(x_variance, y_variance)
    return xq_lst, yq_lst

def threed_data(uc, f1, f2):
    assert f1.shape == f2.shape
    nx, ny, nz = f1.shape
    f1d_1, res1d_1 = fcodes_fast.conv3d_to_1d(f1,uc,1,nx,ny,nz)
    f1d_2, res1d_2 = fcodes_fast.conv3d_to_1d(f2,uc,1,nx,ny,nz)
    xq_lst, yq_lst = make_xycols(f1d_1,f1d_2,res1d_1,res1d_2)
    return xq_lst, yq_lst

def draw_qqplot(xq_lst, yq_lst):
    import matplotlib.pyplot as plt
    fig = plt.figure(figsize=(6,6))
    ax1 = fig.add_subplot(111)
    ibin = -1
    xmax = 0
    ymax = 0
    for x_quantiles, y_quantiles in zip(xq_lst, yq_lst):
        ibin = ibin + 1
        if x_quantiles.size > 0 and xmax < np.max(x_quantiles): xmax = np.max(x_quantiles)
        if y_quantiles.size > 0 and ymax < np.max(y_quantiles): ymax = np.max(y_quantiles)
        ax1.scatter(x_quantiles, y_quantiles, s=10, marker='o', label='bin_'+str(ibin),
               alpha=0.3, edgecolors='none')
    plt.title('Q-Q Plot of Fobs of 6pu5(1st) vs 6pu4(2nd)')
    plt.xlabel('Quantiles of 1st sample')
    plt.ylabel('Quantiles of 2nd sample')
    plt.plot((0, ymax), (0, ymax), 'lightgray')
    plt.legend(loc=0)
    plt.savefig('qqplot.eps',format='eps',dpi=300)
    plt.show()
    plt.close()

def make_quantiles(x, y):
    '''import scipy.stats as stats
    import pylab
    stats.probplot(x, dist='norm',plot=pylab)
    pylab.show()'''
    import statsmodels.api as sm
    import matplotlib.pyplot as plt
    pp_x = sm.ProbPlot(x)
    pp_y = sm.ProbPlot(y)
    from statsmodels.graphics.gofplots import qqplot_2samples
    qqplot_2samples(pp_x, pp_y, line='45')
    plt.show()

def qqplot_simple(x, y, quantiles=None, interpolation='nearest'):
    '''Some insights: 
    https://stats.stackexchange.com/questions/403652/two-sample-quantile-quantile-plot-in-python'''
    import numpy as np
    if quantiles is None:
        quantiles = min(len(x), len(y))
    # Compute quantiles of the two samples
    if isinstance(quantiles, int):
        quantiles = np.linspace(start=0, stop=1, num=int(quantiles))
    else:
        quantiles = np.atleast_1d(np.sort(quantiles))
    x_quantiles = np.quantile(x, quantiles, interpolation=interpolation)
    y_quantiles = np.quantile(y, quantiles, interpolation=interpolation)
    return x_quantiles, y_quantiles

def qqplot(input_map1,input_map2):
    if input_map1.endswith(('.mrc','.map')):
        uc,arr1,_ = iotools.read_map(input_map1)
        uc,arr2,_ = iotools.read_map(input_map2)    
        f1 = np.fft.fftshift(np.fft.fftn(arr1))
        f2 = np.fft.fftshift(np.fft.fftn(arr2))
        #xq_lst, yq_lst = using_3d_data(uc, f1, f2)
        xq_lst, yq_lst = threed_data(uc, f1, f2)
        draw_qqplot(xq_lst, yq_lst)
    
    if input_map1.endswith(('.cif')):
        fobs1, resol1 = pass_cif_sf(input_map1)
        fobs2, resol2 = pass_cif_sf(input_map2)
        xq_lst, yq_lst = make_xycols(fobs1,fobs2,resol1,resol2)
        draw_qqplot(xq_lst, yq_lst)

if (__name__ == "__main__"):
    qqplot()


# cut map at different resolution and output

from __future__ import absolute_import, division, print_function, unicode_literals
from emda import iotools
from emda import restools 
import argparse
import numpy as np
import fcodes_fast
from emda.config import *

cmdl_parser = argparse.ArgumentParser(
description='Lowpass filter the map to given resolution\n')
cmdl_parser.add_argument('-m', '--input_map', required=True, help='Input filename for hfmap1')
cmdl_parser.add_argument('-r', '--res_lst', nargs='+', type=np.float32, required=True, help='Resol.(A) ')
cmdl_parser.add_argument('-v', '--verbose', default=False,
                         help='Verbose output')


def lowpass_map(inputmapname, resol):
    from emda import iotools, restools
    import numpy as np
    uc,arr,origin = iotools.read_map(inputmapname)
    fc = np.fft.fftshift(np.fft.fftn(arr))
    nbin,res_arr,bin_idx = restools.get_resolution_array(uc,fc)
    dist = np.sqrt((res_arr - resol)**2)
    cbin = np.argmin(dist) + 1 # adding 1 because fResArr starts with zero
    fout = restools.cut_resolution(fc,bin_idx,res_arr,cbin)
    iotools.write_mrc(np.real(np.fft.ifftn(np.fft.ifftshift(fout))),
                      "{0}_{1}.{2}".format('lowpass',str(resol),'mrc'),
                      uc,
                      origin)    

def main():
    args = cmdl_parser.parse_args()
    uc,arr,origin = iotools.read_map(args.input_map)
    fc = np.fft.fftshift(np.fft.fftn(arr))
    #np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_mask)))
    nx,ny,nz = fc.shape
    nbin,res_arr,bin_idx = restools.get_resolution_array(uc,fc)
    for resol in args.res_lst:
        #fout = restools.cut_resolution(fc,bin_idx,res_arr,resol)
        fout = restools.cut_resolution(fc,bin_idx,res_arr,resol)
        np.real(np.fft.ifftn(fout))
        iotools.write_mrc(np.real(np.fft.ifftn(np.fft.ifftshift(fout))),
                          "{0}_{1}.{2}".format('lowpass',str(resol),'mrc'),
                          uc,
                          origin)

if(__name__ == "__main__"):
    main()


"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
import sys
import argparse
from emda.config import *
from emda.iotools import read_map,write_mrc,read_mtz
from emda.restools import get_resArr,remove_edge,create_soft_edged_kernel_pxl
from emda.maptools import mtz2map

cmdl_parser = argparse.ArgumentParser(
                                      description='Computes 3D local correlation using maps\n')
cmdl_parser.add_argument('-h1', '--half1_map', required=True, help='Input filename for hfmap1')
cmdl_parser.add_argument('-h2', '--half2_map', required=True, help='Input filename for hfmap2')
cmdl_parser.add_argument('-ml', '--model', required=False, help='Input model MAP/MTZ file')
#cmdl_parser.add_argument('-m', '--mask_map', required=False, help='Input filename for mask')
cmdl_parser.add_argument('-k', '--kernel_size', type=np.int32, required=False, default=5, help='Kernel size (Pixels)')
cmdl_parser.add_argument('-v', '--verbose', default=False,
                         help='Verbose output')

def get_fullmapcor(hfmap_corr):
    import numpy.ma as ma
    hfmap_corr_ma = ma.masked_less_equal(hfmap_corr,0.0)
    ccf_ma = 2 * hfmap_corr_ma / (1.0 + hfmap_corr_ma)
    ccf = ccf_ma.filled(0.0)
    return ccf

def cc_twosimilarmaps(ccmap12,ccmap1,ccmap2,uc,origin):
    import numpy.ma as ma
    ccmap1_ma = ma.masked_less_equal(ccmap1,0.0)
    ccmap2_ma = ma.masked_less_equal(ccmap2,0.0)
    ccmap12_ma = ma.masked_less_equal(ccmap12,0.0)
    cc12_ma = ccmap12_ma * np.sqrt(ccmap1_ma) * np.sqrt(ccmap2_ma)
    cc12 = cc12_ma.filled(0.0)
    #write_mrc(cc12 * cc_mask,'map12_realspacecc.mrc',uc,origin)
    return cc12

def truemap_model_cc(mapmodelcc,fullmapcc):
    import numpy.ma as ma
    from emda.iotools import mask_by_value_greater
    mapmodelcc_ma = ma.masked_less_equal(mapmodelcc,0.3) # To prevent large numbers in truemapmodelcc
    fullmapcc_ma = ma.masked_less_equal(fullmapcc,0.3) # To prevent large numbers in truemapmodelcc
    #truemapmodelcc = mask_by_value_greater(mapmodelcc / np.sqrt(fullmapcc),masking_value=1.0)
    truemapmodelcc_ma = mapmodelcc_ma / np.sqrt(fullmapcc_ma)
    truemapmodelcc = mask_by_value_greater(truemapmodelcc_ma.filled(0.0),masking_value=1.0)
    #write_mrc(truemapmodelcc * cc_mask,'truemapmodel_fancg.mrc',uc,origin)
    return truemapmodelcc

def get_3d_realspcorrelation(half1,half2,kern_sphere):
    # Full map correlation using FFT convolve
    import scipy.signal
    import numpy.ma as ma
    import numpy as np
    loc3_A = scipy.signal.fftconvolve(half1, kern_sphere, "same")
    loc3_A2 = scipy.signal.fftconvolve(half1 * half1, kern_sphere, "same")
    loc3_B = scipy.signal.fftconvolve(half2, kern_sphere, "same")
    loc3_B2 = scipy.signal.fftconvolve(half2 * half2, kern_sphere, "same")
    loc3_AB = scipy.signal.fftconvolve(half1 * half2, kern_sphere, "same")

    cov3_AB = loc3_AB - loc3_A * loc3_B
    var3_A = loc3_A2 - loc3_A**2
    var3_B = loc3_B2 - loc3_B**2
    '''cov3_AB_ma = ma.masked_less_equal(cov3_AB,0.0)
    var3_A_ma = ma.masked_less_equal(var3_A,0.0)
    var3_B_ma = ma.masked_less_equal(var3_B,0.0)
    cc_realsp_ma = cov3_AB_ma / np.sqrt(var3_A_ma * var3_B_ma)
    halfmaps_cc = cc_realsp_ma.filled(0.0)'''
    cov3_AB = np.where(var3_A * var3_B <= 0., 0., cov3_AB)
    var3_A = np.where(var3_A <= 0., 0.1, var3_A)
    var3_B = np.where(var3_B <= 0., 0.1, var3_B)
    halfmaps_cc = cov3_AB / np.sqrt(var3_A * var3_B)
    halfmaps_cc = np.where(cov3_AB <= 0., 0.,halfmaps_cc)

    fullmap_cc = 2 * halfmaps_cc / (1.0 + halfmaps_cc)
    return halfmaps_cc,fullmap_cc

def get_3d_realspmapmodelcorrelation(map,model,kern_sphere):
    mapmodel_cc,_ = get_3d_realspcorrelation(map,model,kern_sphere)
    return mapmodel_cc

def create_mask_from_rscc(fullmapcc,threshold=0.3):
    #from math import sqrt,cos,sin
    import numpy.ma as ma
    '''fullmapcc_low_thresholded = fullmapcc > threshold
    mask = fullmapcc_low_thresholded.astype('int')
    mask = mask/np.count_nonzero(mask)'''
    # soft edged mask
    '''r1 = 0.95
    r0 = 0.91
    inner = fullmapcc >= r1
    outer = fullmapcc >= r0
    norm = 1/np.count_nonzero(outer)
    norm  = 1
    inner_norm = inner * norm
    dist1 = (outer * fullmapcc)
    dist2 = (inner * fullmapcc)
    dist = (dist1 - dist2) * fullmapcc
    dist_ma = ma.masked_less_equal(dist,0.0)
    # cosine decay
    soft_ma = (norm * (1 + np.cos(np.pi * (dist_ma - r0)/(r1 - r0))))/2
    soft = soft_ma.filled(0.0)
    mask = (soft + inner_norm)'''
    r5 = 0.5
    r4 = r5 - 0.01
    r3 = r4 - 0.01
    r2 = r3 - 0.01
    r1 = r2 - 0.01
    r0 = r1 - 0.01
    m5 = fullmapcc >= r5
    m4 = fullmapcc >= r4
    m3 = fullmapcc >= r3
    m2 = fullmapcc >= r2
    m1 = fullmapcc >= r1
    m0 = fullmapcc >= r0
    d5 = m5 * fullmapcc
    d4 = m4 * fullmapcc - d5
    d3 = m3 * fullmapcc - (d5 + d4)
    d2 = m2 * fullmapcc - (d3 + d5 + d4)
    d1 = m1 * fullmapcc - (d2 + d3 + d5 + d4)
    d0 = m0 * fullmapcc - (d1 + d2 + d3 + d5 + d4)
    d5[d5 >= r5] = 1
    d4[d4 >= r4] = r4
    d3[d3 >= r3] = r3
    d2[d2 >= r2] = r2
    d1[d1 >= r1] = r1
    d0[d0 >= r0] = r0
    norm = 1/np.count_nonzero(m0)
    norm = 1
    #mask = (d5 + d4 + d3 + d2 + d1 + d0) * norm
    xx = (d4 + d3 + d2 + d1 + d0)
    dist_ma = ma.masked_less_equal(xx,0.0)
    soft_ma = (norm * (1 + np.cos(np.pi * (dist_ma - r0)/(r5 - r0))))/2
    soft = soft_ma.filled(0.0)
    mask = d5 + soft
    return mask

def calculate_modelmap(model_pdb,dim,map_resol):
    import emda.iotools
    from emda.maptools import mtz2map
    model_bfac = 0 # FSC does not depend on the B factor.
    emda.iotools.run_refmac_sfcalc(model_pdb,map_resol,model_bfac)
    #f_model = pass_mtz('sfcalc_from_crd.mtz',dim)
    arr = mtz2map('sfcalc_from_crd.mtz',dim)
    return arr

def rcc(half1_map, half2_map, kernel_size, model=None, model_resol=None, mask_map=None):
    from emda.iotools import read_map,write_mrc
    from emda.maptools import mtz2map
    from emda import restools
    import emda.maskmap_class
    import emda.fsc
    import fcodes_fast
    print('Calculating 3D correlation between half maps and fullmap. Please wait...')
    uc,half1,origin = read_map(half1_map)
    uc,half2,origin = read_map(half2_map)
    # mask taking into account
    if mask_map is not None:
         _,cc_mask, _ = read_map(mask_map)
    if mask_map is None:
        # creating ccmask from half data
        obj_maskmap = emda.maskmap_class.MaskedMaps()
        obj_maskmap.generate_mask(half1, half2)
        cc_mask = obj_maskmap.mask
    #
    nx,ny,nz = half1.shape
    '''
    fResArr = get_resArr(uc,nx)
    cut_mask = remove_edge(fResArr,fResArr[-1])
    cc_mask = np.zeros(shape=(nx,ny,nz),dtype='int')
    cx, cy, cz = cut_mask.shape
    dx = (nx - cx)//2
    dy = (ny - cy)//2
    dz = (nz - cz)//2
    print(dx,dy,dz)
    cc_mask[dx:dx+cx, dy:dy+cy, dz:dz+cz] = cut_mask'''
    # Creating soft-edged mask
    kern_sphere_soft = restools.create_soft_edged_kernel_pxl(kernel_size) # sphere with radius of n pixles
    write_mrc(kern_sphere_soft,'kern_sphere_soft_smax'+str(kernel_size)+'.mrc',uc,origin)
    # Real space correlation maps
    halfmapscc, fullmapcc = get_3d_realspcorrelation(half1,half2,kern_sphere_soft)
    write_mrc(halfmapscc * cc_mask,'hfmaps_3dcc_smax'+str(kernel_size)+'.mrc',uc,origin)
    write_mrc(fullmapcc * cc_mask,'fullmap_3dcc_smax'+str(kernel_size)+'.mrc',uc,origin)
    # Map-model correlation
    if model is not None:
        fullmap = (half1 + half2) / 2.0
        dim = [nx, ny, nz]
        if model.endswith(('.pdb','.ent')):
            if model_resol is None:
                f_hf1 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(half1)))
                f_hf2 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(half2)))
                nbin,res_arr,bin_idx = restools.get_resolution_array(uc,half1)
                bin_fsc,_,_,_,_,_ = emda.fsc.halfmaps_fsc_variance(f_hf1,f_hf2,bin_idx,nbin)
                # determine map resolution using hfmap FSC
                dist = np.sqrt((bin_fsc - 0.143)**2)
                map_resol = res_arr[np.argmin(dist)]
            else:
                map_resol = model_resol
            model_arr = calculate_modelmap(model,dim,map_resol)
        elif model.lower().endswith(('.mrcs', '.mrc', '.map')):
            _,model_arr,_ = read_map(model)
        elif model.lower().endswith('.mtz'):
            model_arr = mtz2map(model,(nx,ny,nz))
        # need to scale model_arr to fullmap
        #scaled_model = scale_model2map(fullmap, model_arr, uc)
        # normalisation
        nbin,res_arr,bin_idx = restools.get_resolution_array(uc,fullmap)
        fullmap_norm = normalized(map=fullmap,bin_idx=bin_idx,nbin=nbin)
        model_norm = normalized(map=model_arr,bin_idx=bin_idx,nbin=nbin)
        #
        mapmodelcc = get_3d_realspmapmodelcorrelation(fullmap_norm,model_norm,kern_sphere_soft)
        write_mrc(mapmodelcc * cc_mask,'fullmapModel_3dcc.mrc',uc,origin)
        print('Map-model correlation calculated!')
        # truemap-model correlation
        truemapmodelcc = truemap_model_cc(mapmodelcc,fullmapcc)
        write_mrc(truemapmodelcc * cc_mask,'truemapModel_3dcc.mrc',uc,origin)

def scale_model2map(fullmap, model_arr, uc):
    import numpy as np
    from emda import restools
    from emda.scale_maps import scale_twomaps_by_power,transfer_power
    f1 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(fullmap)))
    f2 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(model_arr)))
    nbin,res_arr,bin_idx = restools.get_resolution_array(uc,f1)   
    scale_grid = transfer_power(bin_idx,res_arr,
                                scale_twomaps_by_power(f1,
                                                       f2,
                                                       bin_idx=bin_idx,
                                                       res_arr=res_arr))
    scaled_model = np.real(np.fft.ifftn(np.fft.ifftshift(
                                                       f2 * scale_grid)))
    return scaled_model

def normalized(map,bin_idx=None,nbin=None,uc=None):
    # normalise in resol bins
    from emda import fsc,restools
    if bin_idx is None:
        if uc is not None:
            nbin,res_arr,bin_idx = restools.get_resolution_array(uc,map)
    hf1 = np.fft.fftshift(np.fft.fftn(map))
    _,_,_,totalvar,_,eo = fsc.halfmaps_fsc_variance(hf1,hf1,bin_idx,nbin)
    norm_map = np.real(np.fft.ifftn(np.fft.ifftshift(eo)))
    return norm_map

def main():
    args = cmdl_parser.parse_args()
    # Halfmaps and fullmap correlation calculation
    print('Calculating 3D correlation between half maps and fullmap. Please wait...')
    uc,half1,origin = read_map(args.half1_map)
    uc,half2,origin = read_map(args.half2_map)
    nx,ny,nz = half1.shape
    fResArr = get_resArr(uc,nx)

    cut_mask = remove_edge(fResArr,fResArr[-1])
    cc_mask = np.zeros(shape=(nx,ny,nz),dtype='int')
    cx, cy, cz = cut_mask.shape
    dx = (nx - cx)//2
    dy = (ny - cy)//2
    dz = (nz - cz)//2
    print(dx,dy,dz)
    cc_mask[dx:dx+cx, dy:dy+cy, dz:dz+cz] = cut_mask

    # Creating soft-edged mask
    kern_sphere_soft = create_soft_edged_kernel_pxl(args.kernel_size) # sphere with radius of n pixles
    write_mrc(kern_sphere_soft,'kern_sphere_soft_smax'+str(args.kernel_size)+'.mrc',uc,origin)

    # Real space correlation maps
    halfmapscc, fullmapcc = get_3d_realspcorrelation(half1,half2,kern_sphere_soft)
    write_mrc(halfmapscc * cc_mask,'hfmaps_3dcc_smax'+str(args.kernel_size)+'.mrc',uc,origin)
    write_mrc(fullmapcc * cc_mask,'fullmap_3dcc_smax'+str(args.kernel_size)+'.mrc',uc,origin)
    
    # Create mask using correlation
    #mask = create_mask_from_rscc(fullmapcc*cc_mask,0.91)
    #write_mrc(mask * cc_mask,'mask.mrc',uc,origin)

    # Map-model correlation
    if args.model is not None:
        fullmap = (half1 + half2) / 2.0
        if args.model.lower().endswith(('.mrcs', '.mrc', '.map')):
            _,model,_ = read_map(args.model)
        elif args.model.lower().endswith('.mtz'):
            model = mtz2map(args.model,(nx,ny,nz))
        mapmodelcc = get_3d_realspmapmodelcorrelation(fullmap,model,kern_sphere_soft)
        write_mrc(mapmodelcc * cc_mask,'fullmapModel_3dcc.mrc',uc,origin)
        print('Map-model correlation calculated!')
        # truemap-model correlation
        truemapmodelcc = truemap_model_cc(mapmodelcc,fullmapcc)
        write_mrc(truemapmodelcc * cc_mask,'truemapModel_3dcc.mrc',uc,origin)

    # Two maps with and without ligand (or similar situation e.g. two conformations)
    #ccmap1 = loccc_realspace(half1,half2,kern_sphere)
    #ccmap2 = loccc_realspace(half3,half4,kern_sphere)
    #full1 = (half1 + half2) / 2.0
    #full2 = (half3 + half4) / 2.0
    #ccmap12 = loccc_realspace(full1,full2,kern_sphere)
    #cc12 = cc_twosimilarmaps(ccmap12,ccmap1,ccmap2,uc,origin)'''

if(__name__ == "__main__"):
    main()



"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""
from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
import argparse
import sys
from emda import iotools, restools, fsc,plotter
from emda.config import *

# Fullmap from halfmaps

cmdl_parser = argparse.ArgumentParser(description='Fullmap from halfmaps\n')
cmdl_parser.add_argument('-h1', '--half1', required=True, help='input halfmap1')
cmdl_parser.add_argument('-h2', '--half2', required=True, help='input halfmap2')
cmdl_parser.add_argument('-o', '--outfull', required=False, help='output fullmap')

def half2full(ar1, ar2):
    import numpy as np
    assert ar1.shape == ar2.shape
    hf1 = np.fft.fftn(ar1)
    hf2 = np.fft.fftn(ar2)    
    return np.real(np.fft.ifftn((hf1 + hf2)/2.0))

def main():
    args = cmdl_parser.parse_args()
    uc, ar, org = iotools.read_map(args.half1)
    uc, ar2, org = iotools.read_map(args.half2)
    assert ar.shape == ar2.shape
    outfile = 'fullmap.mrc'
    if args.outfull is not None: outfile=args.outfull
    hf1 = np.fft.fftn(ar)
    hf2 = np.fft.fftn(ar2)
    iotools.write_mrc(np.real(np.fft.ifftn((hf1 + hf2)/2.0)),outfile,uc)
    print('Fullmap was written! ', outfile)


if(__name__ == "__main__"):
    main()




# This module calculates FSC between maps and model
# Author: Rangana Warshamanage
# Created: 2019.06.11

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
from timeit import default_timer as timer
import sys
from emda.iotools import read_mrc
from emda.plotter import *
import argparse
from emda.config import *

#debug_mode = 1 # 0: no debug info, 1: debug

cmdl_parser = argparse.ArgumentParser(description='Computes FSC between model and map\n')
cmdl_parser.add_argument('-h1', '--half1_map', required=True, help='Input filename for hfmap1')
cmdl_parser.add_argument('-h2', '--half2_map', required=True, help='Input filename for hfmap2')
cmdl_parser.add_argument('-af', '--modelf_pdb', required=True, help='Input full atomic model')
cmdl_parser.add_argument('-a1', '--model1_pdb', required=True, help='Input halfmap1 atomic model')
cmdl_parser.add_argument('-r', '--model_resol', type=np.float32, required=False, help='Resolution in Angstrom')
cmdl_parser.add_argument('-s', '--map_size', nargs='+', type=np.int, required=False, help='Map size ')
cmdl_parser.add_argument('-f1', '--hff1_map', required=False, help='Input filename for hfmap1')
cmdl_parser.add_argument('-f2', '--hff2_map', required=False, help='Input filename for hfmap2')
cmdl_parser.add_argument('-v', '--verbose', default=False,
                         help='Verbose output')

def calc_fsc_mrc(hf1,hf2,bin_idx,nbin):
    from emda import fsc
    '''import fcodes_fast
    assert hf1.shape == hf2.shape
    nx,ny,nz = hf1.shape
    _,_,_,_,_,bin_fsc = fcodes_fast.calc_fsc_using_halfmaps(
                hf1,hf2,bin_idx,nbin,1,nx,ny,nz)'''
    bin_fsc,_,_,_,_,_ = fsc.halfmaps_fsc_variance(hf1,hf2,bin_idx,nbin)
    return bin_fsc

'''def calculate_modelmap(model_pdb,dim,map_resol):
    model_bfac = 20 # FSC does not depend on the B factor.
    run_refmac_sfcalc(model_pdb,map_resol,model_bfac)
    modelname = 'sfcalc_from_crd.mtz'
    uc2,df = read_mtz_gemmi(modelname)
    f,h,k,l = get_f_gemmi(df)
    f3d,_ = mtz2mrc(uc2,h,k,l,f,dim[0],dim[1],dim[2])
    _,f_model,_ = read_mrc('mtz2mrc.mrc')
    return f_model

def pass_mtz(mtzfile,dim):
    uc,df = read_mtz_gemmi(mtzfile)
    f,h,k,l = get_f_gemmi(df)
    f3d,_ = mtz2mrc(uc,h,k,l,f,dim[0],dim[1],dim[2])
    _,f_map,_ = read_mrc('mtz2mrc.mrc')
    return uc, f_map'''

def calculate_modelmap(model_pdb,dim,map_resol):
    import emda.iotools
    model_bfac = 0 # FSC does not depend on the B factor.
    emda.iotools.run_refmac_sfcalc(model_pdb,map_resol,model_bfac)
    f_model = pass_mtz('sfcalc_from_crd.mtz',dim)
    return f_model

def pass_mtz(mtzfile,dim):
    from emda.maptools import mtz2map
    import numpy as np
    arr = mtz2map(mtzfile,dim)
    f_map = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr)))
    return f_map

def map_model_fsc(half1_map, half2_map, modelf_pdb, model1_pdb, mask_map=None, map_size=None, model_resol=None):
    import fcodes_fast
    from emda.iotools import read_mrc,read_map
    from emda.plotter import plot_nlines,plot_nlines2
    from emda import restools
    import emda.maskmap_class
    import numpy as np
    fsc_list = []
    ########## FSC between half maps ##########
    # if maps are in MRC format
    if half1_map.endswith(('.mrc','.mrcs','.map')):
        #uc,f_hf1, _ = read_mrc(half1_map)
        uc,arr1, _ = read_map(half1_map)
        #f_hf1 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr1)))
    if half2_map.endswith(('.mrc','.mrcs','.map')):
        #uc,f_hf2, _ = read_mrc(half2_map)
        uc,arr2, _ = read_map(half2_map)
        #f_hf2 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr2)))
    #f_ful = (f_hf1 + f_hf2)/2.0 
    # mask taking into account
    if mask_map is not None:
         uc,msk, _ = read_map(mask_map)
    if mask_map is None:
        # creating ccmask from half data
        obj_maskmap = emda.maskmap_class.MaskedMaps()
        obj_maskmap.generate_mask(arr1, arr2)
        msk = obj_maskmap.mask

    f_hf1 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr1 * msk)))
    f_hf2 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr2 * msk)))
    f_ful = (f_hf1 + f_hf2)/2.0 
    # if maps are in MTZ format
    if half1_map.endswith(('.mtz')):
        if map_size is None:
            print('Need map dimensions.')
            exit()
        dim = map_size
        if len(dim) < 3: 
            print('Need three values space delimited')
            exit()
        if len(dim) > 3:
            dim = dim[:3]
        f_hf1 = pass_mtz(half1_map, dim)
    if half2_map.endswith(('.mtz')):
        f_hf2 = pass_mtz(half2_map, dim)  
    # making resolution grid
    dim = f_hf1.shape
    nx,ny,nz = f_hf1.shape
    nbin,res_arr,bin_idx = restools.get_resolution_array(uc,f_hf1)
    # FSC between halfmaps
    bin_fsc = calc_fsc_mrc(f_hf1,f_hf2,bin_idx,nbin)
    fsc_list.append(bin_fsc)
    ##########################################
    if model_resol is None:
        # determine map resolution using hfmap FSC
        dist = np.sqrt((bin_fsc - 0.143)**2)
        map_resol = res_arr[np.argmin(dist)]
    else:
        map_resol = model_resol

    ########## Calculate maps from models ##########
    # if model is suppied as coordinates
    dim = [nx, ny, nz]
    if modelf_pdb.endswith(('.pdb','.ent')):
        f_modelf = calculate_modelmap(modelf_pdb,dim,map_resol)
    if model1_pdb.endswith(('.pdb','.ent')):
        f_model1 = calculate_modelmap(model1_pdb,dim,map_resol)
    # if model is suppied as maps (for whatever crazy reason)
    if modelf_pdb.endswith(('.mrc','.mrcs','.map')):
        _,f_modelf,_ = read_mrc(modelf_pdb)
    if model1_pdb.endswith(('.mrc','.mrcs','.map')):
        _,f_model1,_ = read_mrc(model1_pdb)
    # if model is suppied as mtz (for whatever crazy reason)
    if modelf_pdb.endswith(('.mtz')):
        f_modelf = pass_mtz(modelf_pdb, dim)
    if model1_pdb.endswith(('.mtz')):
        f_model1 = pass_mtz(model1_pdb, dim)
    ################################################

    ########## FSC between maps and models #########
    # FSC between halfmaps and model1
    for imap in [f_hf1,f_hf2]:
        bin_fsc = calc_fsc_mrc(imap,f_model1,bin_idx,nbin)
        fsc_list.append(bin_fsc)
    # FSC between fullmap and modelfull
    bin_fsc = calc_fsc_mrc(f_ful,f_modelf,bin_idx,nbin)
    fsc_list.append(bin_fsc)
    ################################################
    # output plots
    plot_nlines(res_arr,fsc_list,
                'allmap_fsc_modelvsmap.eps',
                ["hf1-hf2","half1-model1","half2-model1","fullmap-model"])
    plot_nlines2(1/res_arr,fsc_list,
                 'allmap_fsc_modelvsmap-2.eps',
                 ["hf1-hf2","half1-model1","half2-model1","fullmap-model"])



def main():
    import fcodes_fast
    from emda import restools
    args = cmdl_parser.parse_args()
    # Onetime manipulation for Wenjuan maps.
    # we supply unfiltered half maps and use them for
    # hf1-hf2 fsc calculation
    calc_hf_fsc = True
    if args.hff1_map is not None:
        uc,f_hff1, _ = read_mrc(args.hff1_map)
        uc,f_hff2, _ = read_mrc(args.hff2_map)
        # making resolution grid
        nx,ny,nz = f_hff1.shape
        nbin,res_arr,bin_idx = restools.get_resolution_array(uc,f_hff1)
        # FSC between halfmaps
        fsc_list = []
        bin_fsc = calc_fsc_mrc(f_hff1,f_hff2,bin_idx,nbin)
        fsc_list.append(bin_fsc)
        calc_hf_fsc = False

    # if maps are in MTZ format
    if args.half1_map.endswith(('.mtz')):
        if args.map_size is None:
            print('Need map dimensions.')
            exit()
        dim = args.map_size
        if len(dim) < 3: 
            print('Need three values space delimited')
            exit()
        if len(dim) > 3:
            dim = dim[:3]
        f_hf1 = pass_mtz(args.half1_map, dim)
    if args.half2_map.endswith(('.mtz')):
        f_hf2 = pass_mtz(args.half2_map, dim)

    # if maps are in MRC format
    if args.half1_map.endswith(('.mrc','.mrcs','.map')):
        uc,f_hf1, _ = read_mrc(args.half1_map)
    if args.half1_map.endswith(('.mrc','.mrcs','.map')):
        uc,f_hf2, _ = read_mrc(args.half2_map)
    f_ful = (f_hf1 + f_hf2)/2.0

    if calc_hf_fsc:
        # making resolution grid
        nx,ny,nz = f_hf1.shape
        nbin,res_arr,bin_idx = restools.get_resolution_array(uc,f_hf1)
        # FSC between halfmaps
        fsc_list = []
        bin_fsc = calc_fsc_mrc(f_hf1,f_hf2,bin_idx,nbin)
        fsc_list.append(bin_fsc)
    # determine map resolution using hfmap FSC
    dist = np.sqrt((bin_fsc - 0.143)**2)
    map_resol = res_arr[np.argmin(dist)]
    if args.model_resol is not None: map_resol = args.map_resol

    # Calculate maps from models
    # if model is suppied as maps
    modelf_pdb = args.modelf_pdb
    model1_pdb = args.model1_pdb
    if modelf_pdb.endswith(('.mrc','.mrcs','.map')):
        _,f_modelf,_ = read_mrc(args.modelf_pdb)
    if model1_pdb.endswith(('.mrc','.mrcs','.map')):
        _,f_model1,_ = read_mrc(args.modelf_pdb)
    # if model is suppied as coordinates
    dim = [nx, ny, nz]
    if modelf_pdb.endswith(('.pdb','.ent')):
        #pdb2mmcif(args.modelf_pdb)
        f_modelf = calculate_modelmap(args.modelf_pdb,dim,map_resol)
    if model1_pdb.endswith(('.pdb','.ent')):
        #pdb2mmcif(args.model1_pdb)
        f_model1 = calculate_modelmap(args.model1_pdb,dim,map_resol)

    # FSC between halfmaps and model1
    for imap in [f_hf1,f_hf2]:
        bin_fsc = calc_fsc_mrc(imap,f_model1,bin_idx,nbin)
        fsc_list.append(bin_fsc)
    # FSC between fullmap and modelfull
    bin_fsc = calc_fsc_mrc(f_ful,f_modelf,bin_idx,nbin)
    fsc_list.append(bin_fsc)

    # output plots
    plot_nlines(res_arr,fsc_list,'allmap_fsc_modelvsmap.eps',["hf1-hf2","half1-model1","half2-model1","fullmap-model"])
    plot_nlines2(1/res_arr,fsc_list,'allmap_fsc_modelvsmap-2.eps',["hf1-hf2","half1-model1","half2-model1","fullmap-model"])

if (__name__ == "__main__"):
    main()



    
    





"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from emda.mapfit import map_Class
from emda.mapfit import emfit_Class

def output_rotated_maps(emmap1, r_lst, t_lst, Bf_arr=[0.0]):
    import numpy as np
    import fcodes_fast
    from emda.mapfit import utils
    from emda.plotter import plot_nlines,plot_nlines_log
    from scipy.ndimage.interpolation import shift
    from emda import iotools 
    import numpy.ma as ma

    com = emmap1.com1  
    fo_lst = emmap1.fo_lst   
    bin_idx = emmap1.bin_idx  
    nbin = emmap1.nbin     
    res_arr = emmap1.res_arr  
    cell = emmap1.map_unit_cell 
    origin = emmap1.map_origin
    #assert len(fo_lst) == len(t_lst) == len(r_lst)
    nx,ny,nz = fo_lst[0].shape
    frt_lst = []
    frt_lst.append(fo_lst[0])  
    cov_lst = []
    fsc12_lst = []
    fsc12_lst_unaligned = []
    imap_f = 0
    # static map
    data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fo_lst[0]))))
    data2write = shift(data2write, np.subtract(com,emmap1.box_centr))
    iotools.write_mrc(data2write,'static_map.mrc',cell,origin)
    del data2write
    for fo, t, rotmat in zip(fo_lst[1:], t_lst, r_lst):
        _,f1f2_fsc_unaligned = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                    fo_lst[0],fo,bin_idx,nbin,0,nx,ny,nz)
        fsc12_lst_unaligned.append(f1f2_fsc_unaligned)            
        imap_f = imap_f + 1
        st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
        frt_full = utils.get_FRS(cell,rotmat,fo * st)[:,:,:,0]
        frt_lst.append(frt_full)
        data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(frt_full))))
        data2write = shift(data2write, np.subtract(com,emmap1.box_centr))
        iotools.write_mrc(data2write,"{0}_{1}.{2}".format('fitted_map',str(imap_f),'mrc'),cell,origin)
        # estimating covaraince between current map vs. static map
        f1f2_covar,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                    fo_lst[0],frt_full,bin_idx,nbin,0,nx,ny,nz)
        cov_lst.append(f1f2_covar)
        fsc12_lst.append(f1f2_fsc)

        plot_nlines(res_arr,
                    fsc12_lst_unaligned + fsc12_lst,
                    'fsc.eps',
                    ["s-u1","s-a1"])

def main(maplist, ncycles=10, t_init=[0.0, 0.0, 0.0], theta_init=[(1,0,0),0.0], smax=6, masklist=None, mapavg=False):
    if len(maplist) < 2: 
        print(" At least 2 maps required!")
        exit()
    try:
        emmap1 = map_Class.EmmapOverlay(maplist,masklist)
    except NameError:
        emmap1 = map_Class.EmmapOverlay(maplist)
    emmap1.load_maps()
    emmap1.calc_fsc_from_maps()
    # converting theta_init to rotmat for initial iteration
    from quaternions import get_quaternion, get_RM
    import numpy as np
    q = get_quaternion(theta_init)
    q = q/np.sqrt(np.dot(q,q))
    print(q)
    rotmat = get_RM(q)
    t = t_init
    rotmat_lst = []
    transl_lst = []
    # frequency marching test
    from frequency_marching import frequency_marching
    static_bin_lst = []
    #for ibin in [40, 50, 55, 60]: # Takanori ATPase 31-34
    #for ibin in [30, 40, 50, 60, 65]: # Bianka data
    #for ibin in [50,100]: # vinoth-full map
    for ibin in [44,66]: # vinoth-reboxed
        cutmap,cBIdx, cbin = frequency_marching(emmap1.eo_lst[0],
                                        emmap1.bin_idx,
                                        emmap1.res_arr,
                                        bmax=ibin)
        static_bin_lst.append(cutmap)

    for ifit in range(1, len(emmap1.eo_lst)):
        i = -1
        #for ibin in [40, 50, 55, 60]:
        #for ibin in [30, 40, 50, 60, 65]:
        #for ibin in [50,100]:
        for ibin in [44,66]: # vinoth-reboxed
            i = i + 1
            #ceo_lst = []
            cutmap,cBIdx, cbin = frequency_marching(emmap1.eo_lst[ifit],
                                                emmap1.bin_idx,
                                                emmap1.res_arr,
                                                bmax=ibin)
            assert static_bin_lst[i].shape == cutmap.shape
            emmap1.ceo_lst = [static_bin_lst[i],cutmap]
            emmap1.cbin_idx = cBIdx
            emmap1.cdim = cutmap.shape
            emmap1.cbin = cbin
            fit = emfit_Class.EmFit(emmap1)
            fit.minimizer(ncycles, t, rotmat)
            ncycles = 5
            t = fit.t_accum
            rotmat = fit.rotmat
        
        rotmat_lst.append(fit.rotmat)
        transl_lst.append(fit.t_accum)
        # free memory
        del fit

    output_rotated_maps(emmap1,
                        rotmat_lst,
                        transl_lst)


if (__name__ == "__main__"):
    maplist = ['/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/nat_hf2.mrc',
           '/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/lig_hf2.mrc']
    masklist = ['/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/nat_msk.mrc',
            '/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/lig_msk.mrc']
    main(maplist)





# Neighbor averaging method based fake half map
def fakehalf(f_map, knl=3):
    import numpy as np
    import scipy.signal
    #kernel = np.ones((knl, knl, knl), dtype=int)
    #kernel[1,1,1] = 0
    #kernel = kernel/np.sum(kernel)
    box_size = knl
    kernel = np.ones((box_size, box_size, box_size)) / (box_size ** 3)
    kernel[1,1,1] = 0
    kernel = kernel/np.sum(kernel)
    fakehalf = scipy.signal.fftconvolve(f_map, kernel, "same")
    return fakehalf


"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""
from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
from timeit import default_timer as timer
from emda.plotter import plot_nlines
from emda.quaternions import get_RM, rotationMatrixToEulerAngles
from emda.mapfit.utils import get_FRS,create_xyz_grid,get_xyz_sum
import fcodes_fast
from emda.restools import cut_resolution_for_linefit
from emda.mapfit.derivatives_newmethod2 import derivatives
from emda.config import *

np.set_printoptions(suppress=True) # Suppress insignificant values for clarity

class LineFit:
    def __init__(self):
        self.e0_lf          = None
        self.step           = None
        self.e1_lf          = None
        self.cbin_idx_lf    = None
        self.cbin_lf        = None
        self.w_grid         = None


    def get_linefit_static_data(self,e0,cbin_idx,res_arr,smax):
        self.e0_lf,self.cbin_idx_lf,self.cbin_lf = cut_resolution_for_linefit(e0,
                                                    cbin_idx,
                                                    res_arr,
                                                    smax)

    def f(self,k):
        from emda.mapfit.utils import get_fsc_wght
        # w = 1.0 for line search
        nx,ny,nz = self.e0_lf.shape
        t = self.step[:3]*k[0]
        st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
        q_init = np.array([1.0, 0.0, 0.0, 0.0])
        tmp = np.insert(self.step[3:]*k[1], 0, 0.0)
        tmp = tmp + q_init
        q = tmp/np.sqrt(np.dot(tmp, tmp))
        rotmat = get_RM(q)
        ers = get_FRS(rotmat,self.e1_lf * st, interp='linear')
        w_grid = get_fsc_wght(self.e0_lf, ers[:,:,:,0],self.cbin_idx_lf,self.cbin_lf)
        fval = np.sum(w_grid * self.e0_lf * np.conjugate(ers[:,:,:,0]))
        return -fval.real

    def calc_fval_for_different_kvalues_at_this_step(self,e1):
        from scipy import optimize
        start = timer()
        nx,ny,nz = e1.shape
        cx = self.e0_lf.shape[0] // 2
        cy = cz = cx
        dx = int((nx - 2*cx)/2)
        dy = int((ny - 2*cy)/2)
        dz = int((nz - 2*cz)/2)
        self.e1_lf = e1[dx:dx+2*cx,dy:dy+2*cy,dz:dz+2*cz]
        assert self.e1_lf.shape == self.e0_lf.shape
        init_guess = [1.0,1.0]
        minimum = optimize.minimize(self.f, init_guess, method='Powell')
        end = timer()
        #print(' time for line search: ', end-start)
        return minimum.x


"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from emda.mapfit import map_Class
from emda.mapfit import emfit_Class
from emda.config import *

def output_rotated_maps(emmap1, r_lst, t_lst, Bf_arr=[0.0]):
    import numpy as np
    import fcodes_fast
    from emda.mapfit import utils
    from emda.plotter import plot_nlines,plot_nlines_log
    from scipy.ndimage.interpolation import shift
    from emda import iotools
    import numpy.ma as ma
    from emda import fsc

    com = emmap1.com1  
    fo_lst = emmap1.fo_lst   
    eo_lst = emmap1.eo_lst
    hffsc_lst = emmap1.hffsc_lst 
    signalvar_lst = emmap1.signalvar_lst
    totalvar_lst = emmap1.totalvar_lst
    bin_idx = emmap1.bin_idx  
    nbin = emmap1.nbin     
    res_arr = emmap1.res_arr  
    cell = emmap1.map_unit_cell 
    origin = emmap1.map_origin
    #assert len(fo_lst) == len(t_lst) == len(r_lst)
    nx,ny,nz = fo_lst[0].shape
    frt_lst = []
    frt_lst.append(fo_lst[0])  
    ert_lst = []
    ert_lst.append(eo_lst[0])  
    cov_lst = []
    fsc12_lst = []
    imap_f = 0
    # static map
    data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fo_lst[0]))))
    data2write = shift(data2write, np.subtract(com,emmap1.box_centr))
    iotools.write_mrc(data2write,'static_map.mrc',cell,origin)
    del data2write
    for fo, eo, t, rotmat in zip(fo_lst[1:], eo_lst[1:], t_lst, r_lst):
        imap_f = imap_f + 1
        st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
        frt_full = utils.get_FRS(cell,rotmat,fo * st)[:,:,:,0]
        frt_lst.append(frt_full)
        data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(frt_full))))
        data2write = shift(data2write, np.subtract(com,emmap1.box_centr))
        iotools.write_mrc(data2write,
                          "{0}_{1}.{2}".format('fitted_map',str(1),'mrc'),
                          cell,
                          origin)
        # estimating covaraince between current map vs. static map
        f1f2_fsc,f1f2_covar = fsc.anytwomaps_fsc_covariance(fo_lst[0],
                                                            frt_full,
                                                            bin_idx,
                                                            nbin)
        '''f1f2_covar,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                                fo_lst[0],frt_full,bin_idx,nbin,0,nx,ny,nz)'''
        # mask f1f2_covar so that anything beyond zero get zeros
        f1f2_covar = set_array(f1f2_covar)
        cov_lst.append(f1f2_covar)
        fsc12_lst.append(f1f2_fsc)
        # normalised maps
        ert_lst.append(utils.get_FRS(cell,rotmat,eo * st)[:,:,:,0])

    # Here calculates the new singal_vars using covariance
    static_signal = np.zeros((nbin,len(cov_lst)), dtype='float')
    pred_signalvar_lst = []
    # 1. fit a curve to S-matrix and get B value of that curve
    for i in range(len(cov_lst)):
        s_0i_ma = ma.masked_less_equal(cov_lst[i],0.0)
        # apply s_0i mask on sgnl_var_0i
        sgnl_var_0i_ma = ma.masked_less_equal(signalvar_lst[0] * 
                                              signalvar_lst[i+1], 
                                              0.0)
        sgnl_var_0i_ma = ma.masked_where(ma.getmask(s_0i_ma), sgnl_var_0i_ma)
        s_0i = (s_0i_ma/ma.masked_where(ma.getmask(s_0i_ma), 
                        np.sqrt(sgnl_var_0i_ma.filled(0.0)))).filled(0.0)
        bfac, intercept = get_bfac(res_arr,s_0i)
        s = ((1.0/res_arr)**2)/4.0
        pred_s01 =  np.exp(-1.0*abs(bfac) * s + intercept)
        plot_nlines(res_arr,[s_0i, pred_s01],'s_0i.eps',["s_0i","pred_s_0i"])        
        # 2. estimate scale and relative B value between maps
        resol = get_resol(fsc12_lst[i],res_arr)
        scale, bfac_relative = estimate_scale_and_relativebfac([frt_lst[0],frt_lst[i+1]],cell,resol)
        # 3. now predict new signalvar of ii maps using cov_ij
        signal_00 = cov_lst[0]*np.exp(abs(bfac)*s) * np.sqrt(scale*np.exp(bfac_relative*s))
        static_signal[:,i] = signal_00
        if i == 0:
            pred_signalvar_lst.append(signal_00)
        signal_11 = cov_lst[0]*np.exp(abs(bfac)*s) / np.sqrt(scale*np.exp(bfac_relative*s))
        pred_signalvar_lst.append(signal_11)
        # 4. now plot new and old signals
        '''signal_lst2plot = [ signalvar_lst[0],
                            signalvar_lst[i+1],
                            cov_lst[i],
                            signal_00,
                            signal_11 ]
        plot_nlines_log(res_arr,
                        signal_lst2plot,
                        ["old_s00","old_s11","s01","new_s00","new_s11"],
                        'log_variance_all.eps')'''

    pred_signalvar_lst[0] = np.average(static_signal, axis=1)
    plot_nlines_log(res_arr,
                    pred_signalvar_lst)    

    # output sigma values into a file
    '''for i in range(len(signal_00)):
        s1 = signal_00[i]
        s2 = signal_11[i]
        s12 = cov_lst[0][i]
        print(s1, s2, s12, s12**2 - s1*s2)'''

    #pred_signalvar_lst = [signal_00,signal_11]
    # map averaging with normalised maps
    averagemaps = utils.avg_and_diffmaps(ert_lst,
                                         cell,nbin,
                                         pred_signalvar_lst,
                                         totalvar_lst,
                                         cov_lst,
                                         #fsc12_lst,
                                         hffsc_lst,
                                         bin_idx,
                                         emmap1.s_grid,
                                         res_arr,
                                         Bf_arr)
    # calc. fsc between static map and averaged maps
    for i in range(2):
        f1f2_fsc,_ = fsc.anytwomaps_fsc_covariance(eo_lst[0],
                                                   averagemaps[:,:,:,i,0],
                                                   bin_idx,
                                                   nbin)
        '''_,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                            eo_lst[0],averagemaps[:,:,:,i,0],bin_idx,nbin,0,nx,ny,nz)'''
        fsc12_lst.append(f1f2_fsc)
    '''plot_nlines(res_arr,
                fsc12_lst,
                'fsc_compare.eps',
                ["static-aligned2","static-avg1","static-avg2"])     ''' 
    #utils.output_maps(averagemaps,emmap1.com1,cell,origin,Bf_arr)


'''def difference_map(emmap1, r_lst, t_lst, Bf_arr=[0.0]):
    # calculate only the difference map
    from emda.mapfit import diffmap
    #diffmap.calc_diffmap(emmap1, r_lst, t_lst, Bf_arr=[0.0])
    diffmap.calc_diffmap_simple(emmap1, r_lst, t_lst, Bf_arr=[0.0])'''

def get_resol(bin_fsc,res_arr):
    import numpy as np
    bin_fsc = bin_fsc[bin_fsc > 0.1]
    if len(bin_fsc) > 0: dist = np.sqrt((bin_fsc - 0.143)**2)
    return res_arr[np.argmin(dist)]

def get_bfac(res_arr,signal,scale=1):
    from scipy import stats
    import numpy as np
    high_res = 5.5 # Angstrom. This is a hard cutoff. Need to change later
    res_ub = np.sqrt((res_arr - high_res)**2)
    ub_res = np.argmin(res_ub)
    res_arr_trunc, signal_trunc = trunc_array(res_arr,signal)
    if len(signal_trunc) < ub_res:
        s = 1/res_arr[:ub_res]
        slope, intercept,_,_,_ = stats.linregress((s*s)/4.0, np.log(signal[:ub_res]))
    else:
        s = 1/res_arr_trunc
        slope, intercept,_,_,_ = stats.linregress((s*s)/4.0, np.log(signal_trunc))
    bfac = slope
    print('B factor = ', bfac, 'Intercept = ', intercept)    
    return bfac, intercept

def estimate_scale_and_relativebfac(fo_lst,uc,resol):
    from emda.mapfit import curve_fit_3
    params = curve_fit_3.main(fo_lst,uc,resol)
    scale, bfac_relative = params
    return scale, bfac_relative

def set_array(arr):
    set2zero = False
    i = -1
    for ival in arr:
        i = i + 1
        if ival <= 0.0 and not set2zero:
            set2zero = True
        if set2zero: arr[i] = 0.0
    return arr

def trunc_array(res_arr,signal):
    import numpy as np
    val_lst = []
    res_lst = []
    j = -1
    fsc_thresh = 0.7 # this is an arbitrary choice.
    for ival in signal:
        if ival < 1.0:
            if ival > fsc_thresh:
                j = j + 1
                val_lst.append(ival)
                res_lst.append(res_arr[j])
            if ival < fsc_thresh:
                break
    return np.array(res_lst, dtype='float', copy=False), np.array(val_lst, dtype='float', copy=False)

def fsc_between_static_and_transfomed_map(staticmap,movingmap,bin_idx,rm,t,cell,nbin):
    import fcodes_fast
    from emda.mapfit import utils
    from emda import fsc
    nx, ny, nz = staticmap.shape
    st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
    frt_full = utils.get_FRS(cell,rm,movingmap * st)[:,:,:,0]
    f1f2_fsc,_ = fsc.anytwomaps_fsc_covariance(staticmap,
                                               frt_full,
                                               bin_idx,
                                               nbin)
    '''f1f2_covar,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                staticmap,frt_full,bin_idx,nbin,0,nx,ny,nz)'''
    return f1f2_fsc

def get_ibin(bin_fsc):
    import numpy as np
    bin_fsc = bin_fsc[bin_fsc > 0.1]
    #dist = np.sqrt((bin_fsc - 0.143)**2)
    dist = np.sqrt((bin_fsc - 0.3)**2)
    ibin = np.argmin(dist) + 1 # adding 1 because fResArr starts with zero
    return ibin

def main(maplist, ncycles, t_init, theta_init, smax, masklist, fobj, interp):
    from emda.quaternions import get_quaternion, get_RM
    import numpy as np
    from emda.mapfit.frequency_marching import frequency_marching
    from emda.mapfit import newsignal
    from emda.mapfit import run_fit

    if len(maplist) < 4: 
        print('Map averaging requires at least two sets of maps ' 
             'with half sets')
        exit()
    elif (len(maplist) >= 4 and len(maplist) % 2 == 0):
        print('Map average enabled')
        fobj.write('Map average enabled\n')
    try:
        emmap1 = map_Class.EmmapAverage(maplist,masklist)
    except NameError:
        emmap1 = map_Class.EmmapAverage(maplist)
    emmap1.load_maps(fobj)
    emmap1.calc_fsc_variance_from_halfdata(fobj)
    # converting theta_init to rotmat for initial iteration
    fobj.write('\n')
    fobj.write('Initial fitting parameters:\n')
    fobj.write('    Translation: '+ str(t_init)+' \n')
    fobj.write('    Rotation: '+ str(theta_init)+' \n')
    q = get_quaternion(theta_init)
    q = q/np.sqrt(np.dot(q,q))
    print('Initial quaternion: ', q)
    rotmat = get_RM(q)
    fobj.write('    Rotation matrix: '+ str(rotmat)+' \n')
    fobj.write('\n')
    fobj.write('    # fitting cycles: '+ str(ncycles)+' \n')
    t = t_init
    rotmat_lst = []
    transl_lst = []
    # resolution estimate for line-fit
    dist = np.sqrt((emmap1.res_arr - smax)**2) # smax can be a reasonable value
    slf = np.argmin(dist) + 1 
    if slf % 2 != 0: slf = slf - 1
    slf = min([len(dist), slf])
    #
    for ifit in range(1, len(emmap1.eo_lst)):
        fobj.write('Fitting set#: ' + str(ifit)+' \n')
        rotmat, t = run_fit.run_fit(emmap1,smax,rotmat,t,slf,ncycles,ifit,fobj,interp)
        '''for i in range(5):
            if i==0:
                #smax = 6.0 # A
                dist = np.sqrt((emmap1.res_arr - smax)**2)
                ibin = np.argmin(dist) + 1 # adding 1 because fResArr starts with zero
                ibin_old = ibin
                f1f2_fsc = fsc_between_static_and_transfomed_map(
                                            emmap1.fo_lst[0],
                                            emmap1.fo_lst[ifit],
                                            emmap1.bin_idx,
                                            rotmat,
                                            t,
                                            emmap1.map_unit_cell,
                                            emmap1.nbin)
                fsc_lst.append(f1f2_fsc)
            else:
                # Apply initial rotation and translation to calculate fsc
                f1f2_fsc = fsc_between_static_and_transfomed_map(
                                            emmap1.fo_lst[0],
                                            emmap1.fo_lst[ifit],
                                            emmap1.bin_idx,
                                            rotmat,
                                            t,
                                            emmap1.map_unit_cell,
                                            emmap1.nbin)
                ibin = get_ibin(f1f2_fsc)
                if ibin_old == ibin: 
                    fsc_lst.append(f1f2_fsc)
                    fobj.write('\n')
                    fobj.write('FSC between static and moving maps\n')
                    fobj.write('\n')
                    fobj.write('bin#\n')
                    fobj.write('resolution (A)\n')
                    fobj.write('start FSC\n')
                    fobj.write('end FSC\n')
                    fobj.write('\n')
                    for j in range(len(emmap1.res_arr)):
                        #print(emmap1.res_arr[j], fsc_lst[0][j], fsc_lst[1][j])
                        fobj.write("{:5d} {:6.2f} {:8.4f} {:8.4f}\n".format(
                            j,
                            emmap1.res_arr[j],
                            fsc_lst[0][j],
                            fsc_lst[1][j]))
                    break
                else: 
                    ibin_old = ibin

            static_cutmap,cBIdx, cbin = frequency_marching(emmap1.eo_lst[0],
                                            emmap1.bin_idx,
                                            emmap1.res_arr,
                                            bmax=ibin,
                                            fobj=fobj)  
            moving_cutmap,_, _ = frequency_marching(emmap1.eo_lst[ifit],
                                                emmap1.bin_idx,
                                                emmap1.res_arr,
                                                bmax=ibin)
            assert static_cutmap.shape == moving_cutmap.shape
            emmap1.ceo_lst = [static_cutmap,moving_cutmap]
            emmap1.cbin_idx = cBIdx
            emmap1.cdim = moving_cutmap.shape
            emmap1.cbin = cbin
            fit = emfit_Class.EmFit(emmap1)
            fit.minimizer(ncycles, t, rotmat, fobj=fobj)
            ncycles = 3
            t = fit.t_accum
            rotmat = fit.rotmat'''

        #rotmat_lst.append(fit.rotmat)
        #transl_lst.append(fit.t_accum)
        rotmat_lst.append(rotmat)
        transl_lst.append(t)
    #exit()

    '''output_rotated_maps(emmap1,
                        rotmat_lst,
                        transl_lst)'''
    '''newsignal.output_rotated_maps(emmap1,
                        rotmat_lst,
                        transl_lst,
                        fobj=fobj)'''
    '''newsignal.output_rotated_maps_using_fo(emmap1,
                        rotmat_lst,
                        transl_lst,
                        fobj=fobj)'''
    newsignal.output_rotated_maps_using_fo_allmaps(emmap1,
                        rotmat_lst,
                        transl_lst,
                        fobj=fobj)
 

def run_fit(emmap1, f1, f2, t_init, theta_init, smax, ncycles, fobj):
    import numpy as np
    from emda.mapfit.frequency_marching import frequency_marching
    from emda.quaternions import get_quaternion, get_RM
    fsc_lst = []
    q = get_quaternion(theta_init)
    q = q/np.sqrt(np.dot(q,q))
    print(q)
    rotmat = get_RM(q)
    fobj.write('    Rotation matrix: '+ str(rotmat)+' \n')
    fobj.write('\n')
    fobj.write('    # fitting cycles: '+ str(ncycles)+' \n')
    t = t_init
    fobj.write('Fitting set#:  \n')
    for i in range(5):
        if i==0:
            #smax = 6.0 # A
            dist = np.sqrt((emmap1.res_arr - smax)**2)
            ibin = np.argmin(dist) + 1 # adding 1 because fResArr starts with zero
            ibin_old = ibin
            f1f2_fsc = fsc_between_static_and_transfomed_map(
                                        f1,
                                        f2,
                                        emmap1.bin_idx,
                                        rotmat,
                                        t,
                                        emmap1.map_unit_cell,
                                        emmap1.nbin)
            fsc_lst.append(f1f2_fsc)
        else:
            # Apply initial rotation and translation to calculate fsc
            f1f2_fsc = fsc_between_static_and_transfomed_map(
                                        f1,
                                        f2,
                                        emmap1.bin_idx,
                                        rotmat,
                                        t,
                                        emmap1.map_unit_cell,
                                        emmap1.nbin)
            ibin = get_ibin(f1f2_fsc)
            if ibin_old == ibin: 
                fsc_lst.append(f1f2_fsc)
                fobj.write('\n')
                fobj.write('FSC between static and moving maps\n')
                fobj.write('\n')
                fobj.write('bin#\n')
                fobj.write('resolution (A)\n')
                fobj.write('start FSC\n')
                fobj.write('end FSC\n')
                fobj.write('\n')
                for j in range(len(emmap1.res_arr)):
                    #print(emmap1.res_arr[j], fsc_lst[0][j], fsc_lst[1][j])
                    fobj.write("{:5d} {:6.2f} {:8.4f} {:8.4f}\n".format(
                        j,
                        emmap1.res_arr[j],
                        fsc_lst[0][j],
                        fsc_lst[1][j]))
                break
            else: 
                ibin_old = ibin
        static_cutmap,cBIdx, cbin = frequency_marching(f1,
                                                       emmap1.bin_idx,
                                                       emmap1.res_arr,
                                                       bmax=ibin,
                                                       fobj=fobj)  
        moving_cutmap,_, _ = frequency_marching(f2,
                                                emmap1.bin_idx,
                                                emmap1.res_arr,
                                                bmax=ibin)
        assert static_cutmap.shape == moving_cutmap.shape
        emmap1.ceo_lst = [static_cutmap,moving_cutmap]
        emmap1.cbin_idx = cBIdx
        emmap1.cdim = moving_cutmap.shape
        emmap1.cbin = cbin
        fit = emfit_Class.EmFit(emmap1)
        fit.minimizer(ncycles, t, rotmat, fobj=fobj)
        ncycles = 3
        t = fit.t_accum
        rotmat = fit.rotmat
    return fit.rotmat, fit.t_accum


if (__name__ == "__main__"):
    maplist = ['/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/nat_hf1.mrc',
            '/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/nat_hf2.mrc',
            '/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/lig_hf1.mrc',
            '/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/lig_hf2.mrc']
    masklist = ['/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/nat_msk.mrc',
                '/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/nat_msk.mrc',
                '/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/lig_msk.mrc',
                '/Users/ranganaw/MRC/REFMAC/Vinoth/reboxed_maps/lig_msk.mrc']
    main(maplist)





from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
from emda.quaternions import derivatives_wrt_q
from timeit import default_timer as timer

debug_mode = 0

def dFs2(mapin,xyz,vol):
    import numpy as np
    nx,ny,nz = mapin.shape
    #start = timer()
    #tp2 = (2.0 * np.pi)**2 
    #n = -1
    dfs_arr = np.zeros(shape=(nx,ny,nz,3),dtype=np.complex64)
    '''ddfs_arr = np.zeros(shape=(nx,ny,nz,6),dtype=np.complex64)'''
    # Calculating dFC/ds using FFT - 1st derivative using eq. 15
    for i in range(3):  
        dfs = (1/vol) * 2j * np.pi * np.fft.fftn(mapin * xyz[i])
        dfs_arr[:,:,:,i] = np.fft.fftshift(dfs)
        # Calculating ddFC/ds using FFT - 2nd derivative using eq. 17
        '''for j in range(3):
            if i == 0:
                ddfs = -(tp2/vol) * np.fft.fftn(mapin * xyz[i] * xyz[j])
            elif i > 0 and j >= i:
                ddfs = -(tp2/vol) * np.fft.fftn(mapin * xyz[i] * xyz[j])
            else:
                continue
            n = n + 1
            ddfs_arr[:,:,:,n] = np.fft.fftshift(ddfs) '''   
    #end = timer() 
    #print('time for dfs and ddfs calculation: ', end-start)
    return dfs_arr#,ddfs_arr

def derivatives(e0,e1,w_grid,w2_grid,q,sv,xyz,xyz_sum,vol):
    import numpy as np
    import sys
    df_val = np.zeros(6,dtype=np.float64)
    ddf_val = np.zeros(shape=(6,6),dtype=np.float64)
    tp2 = (2.0 * np.pi)**2
    # Translation derivatives
    start = timer()
    for i in range(3):
        # 1st derivative. eq. 33
        df_tmp = np.sum(w_grid * e0 * 
                np.conjugate(e1 * (2.0 * np.pi * 1j) * sv[i])) 
        df_val[i] = np.real(df_tmp)
        for j in range(3):
            # estimated 2nd derivative
            if i == 0:
                ddf_tmp = -tp2 * np.sum(w2_grid * sv[i] * sv[j]) 
            elif i > 0 and j >= i:
                ddf_tmp = -tp2 * np.sum(w2_grid * sv[i] * sv[j]) 
            else:
                ddf_val[i,j] = ddf_val[j,i]
            ddf_val[i,j] = np.real(ddf_tmp)
    
    # Rotation derivatives
    dRdq = derivatives_wrt_q(q) # give current q - vector
    # impose current rotation and translation on F2
    start_dfs2 = timer()
    dFRs = dFs2(np.real(np.fft.ifftn(np.fft.ifftshift(e1))),xyz,vol)
    end_dfs2 = timer()
    print('     time for DFS calc: ', end_dfs2-start_dfs2)
    a = np.zeros(shape=(3,3),dtype=np.float64)
    b = np.zeros(shape=(3,3),dtype=np.float64)
    start_dfr = timer()
    for i in range(3):
        a[:,:] = 0.0
        for k in range(3):
            for l in range(3):
                if k == 0:
                    tmp1 = np.sum(w_grid * np.conjugate(e0) * 
                    (dFRs[:,:,:,k] * sv[l] * dRdq[i,k,l]))
                elif k > 0 and l >= k:
                    tmp1 = np.sum(w_grid * np.conjugate(e0) * 
                    (dFRs[:,:,:,k] * sv[l] * dRdq[i,k,l]))
                else:
                    a[k,l] = a[l,k]
                a[k,l] = tmp1.real
        df_val[i+3] = np.sum(a) # df_val[3] to df_val[5]
    end_dfr = timer()
    print('     time for dfr: ', end_dfr-start_dfr)
    wfsc = w_grid * np.conjugate(e0) * e1 # THIS WORKS
    start_ddfr = timer()
    for i in range(3):
        for j in range(3):
            if i == 0:
                b[:,:] = 0.0
                n = -1
                for k in range(3):
                    for l in range(3): 
                        if k == 0:
                            n = n + 1
                            #tmp2 = np.sum(w_grid * np.conjugate(e0) * 
                            #    (ddfrs_all[:,:,:,n] * sv[k] * sv[l] * 
                            #    dRdq[i,k,l] * dRdq[j,k,l]))
                            tmp2 = -(tp2/vol) * xyz_sum[n] * np.sum(wfsc * sv[k] * sv[l] *
                                dRdq[i,k,l] * dRdq[j,k,l])
                        elif k > 0 and l >= k:
                            n = n + 1
                            #tmp2 = np.sum(w_grid * np.conjugate(e0) * 
                            #    (ddfrs_all[:,:,:,n] * sv[k] * sv[l] * 
                            #    dRdq[i,k,l] * dRdq[j,k,l]))
                            tmp2 = -(tp2/vol) * xyz_sum[n] * np.sum(wfsc * sv[k] * sv[l] *
                                dRdq[i,k,l] * dRdq[j,k,l])
                        else:
                            b[k,l] = b[l,k]
                        b[k,l] = tmp2.real
                ddf_val[i+3,j+3] = np.sum(b) # ddf_val[3] to [5] in i and j indices
            elif i > 0 and j >= i:
                b[:,:] = 0.0
                n = -1
                for k in range(3):
                    for l in range(3): 
                        if k == 0:
                            n = n + 1
                            #tmp2 = np.sum(w_grid * np.conjugate(e0) * 
                            #    (ddfrs_all[:,:,:,n] * sv[k] * sv[l] * 
                            #    dRdq[i,k,l] * dRdq[j,k,l]))
                            tmp2 = -(tp2/vol) * xyz_sum[n] * np.sum(wfsc * sv[k] * sv[l] *
                                dRdq[i,k,l] * dRdq[j,k,l])
                        elif k > 0 and l >= k:
                            n = n + 1
                            #tmp2 = np.sum(w_grid * np.conjugate(e0) * 
                            #    (ddfrs_all[:,:,:,n] * sv[k] * sv[l] * 
                            #    dRdq[i,k,l] * dRdq[j,k,l]))
                            tmp2 = -(tp2/vol) * xyz_sum[n] * np.sum(wfsc * sv[k] * sv[l] *
                                dRdq[i,k,l] * dRdq[j,k,l])
                        else:
                            b[k,l] = b[l,k]
                        b[k,l] = tmp2.real
                ddf_val[i+3,j+3] = np.sum(b) # ddf_val[3] to [5] in i and j indices
            else:
                ddf_val[i+3,j+3] = ddf_val[j+3,i+3]    
    #print(ddf_val)
    # Mixed derivatives
    # NEED A REVIEW
    #for i in range(3):
    #    for j in range(3):
    #        a[:,:] = 0.0
    #        for k in range(3):
    #            for l in range(3):
    #                tmp1 = np.sum(w_grid * np.conjugate(e0) *
    #                    (dFRs[:,:,:,k] * sv[l] * sv[j] * (2.0 * np.pi * 1j) * dRdq[i,k,l]))
    #                a[k,l] = tmp1.real
    #                #print(tmp1)
    #        ddf_val[i,j+3] = np.sum(a) 
    #        ddf_val[i+3,j] = np.sum(a)
    #print(np.linalg.det(ddf_val))
    end_ddfr = timer()
    print('     time for ddfr: ', start_ddfr-end_ddfr)
    end = timer()
    print(' time for derivative calculation: ', end-start)
    #print('df: ', df_val)
    #print('ddf: ', ddf_val)
    if np.linalg.cond(ddf_val) < 1/sys.float_info.epsilon:
        ddf_val_inv = np.linalg.pinv(ddf_val)
    else:
        print('Derivative matrix is non-invertible! Stopping now...')
        exit()
    step = ddf_val_inv.dot(-df_val)
    return step,df_val,e1

def new_dFs2(mapin,xyz,vol):
    import numpy as np
    nx,ny,nz = mapin.shape
    dfs = np.zeros(shape=(nx,ny,nz,3),dtype=np.complex64)
    for i in range(3):  
        dfs[:,:,:,i] = np.fft.fftshift(
                                      (1/vol) * 2j * np.pi * 
                                      np.fft.fftn(mapin * xyz[i]))
    return dfs

def new_derivatives(e0,e1,w_grid,w2_grid,q,sv,xyz,xyz_sum,vol,dfrs=None):
    import numpy as np
    import sys
    import fcodes_fast
    nx, ny, nz = e0.shape
    start = timer()
    sv_np = np.zeros((nx,ny,nz,3), dtype=np.float64)
    for i in range(3):
        sv_np[:,:,:,i] = sv[i]
    dRdq = derivatives_wrt_q(q)
    if dfrs is None:
        print('DFRs calculatation!')
        start = timer()
        dFRs = new_dFs2(np.real(np.fft.ifftn(np.fft.ifftshift(e1))),xyz,vol) 
        end = timer()
        print(' time for dFRs calculation: ', end-start)    
    if dfrs is not None:
        print('Interpolated DFRs are used!')
        dFRs = dfrs
    start = timer()
    df_val,ddf_val = fcodes_fast.calc_derivatives(e0,e1,w_grid,w2_grid,sv_np,dFRs,dRdq,xyz_sum,vol,nx,ny,nz)
    end = timer()
    print(' time for derivative calculation: ', end-start)
    if np.linalg.cond(ddf_val) < 1/sys.float_info.epsilon:
        ddf_val_inv = np.linalg.pinv(ddf_val)
    else:
        print('Derivative matrix is non-invertible! Stopping now...')
        exit()
    step = ddf_val_inv.dot(-df_val)
    return step,df_val

def optimized_derivative_calc(e0,e1,w_grid,w2_grid,q,sv,xyz,xyz_sum,vol):
    import numpy as np
    import sys
    df_val = np.zeros(6,dtype=np.float64)
    ddf_val = np.zeros(shape=(6,6),dtype=np.float64)
    tp2 = (2.0 * np.pi)**2
    # Translation derivatives
    start = timer()
    for i in range(3):
        df_val[i] = np.real(np.sum(w_grid * e0 * 
                np.conjugate(e1 * (2.0 * np.pi * 1j) * sv[i]))) 
        for j in range(3):
            if i == 0 or (i > 0 and j >= i):
                ddf_val[i,j] = -tp2 * np.sum(w2_grid * sv[i] * sv[j])
            else:
                ddf_val[i,j] = ddf_val[j,i]
    
    # Rotation derivatives
    dRdq = derivatives_wrt_q(q) # give current q - vector
    start_dfs2 = timer()
    dFRs = dFs2(np.real(np.fft.ifftn(np.fft.ifftshift(e1))),xyz,vol)
    end_dfs2 = timer()
    print('     time for DFS calc: ', end_dfs2-start_dfs2)
    a = np.zeros(shape=(3,3),dtype=np.float64)
    b = np.zeros(shape=(3,3),dtype=np.float64)
    start_dfr = timer()
    for i in range(3):
        a[:,:] = 0.0
        for k in range(3):
            for l in range(3):
                if k == 0 or (k > 0 and l >= k):
                    a[k,l] = np.real(np.sum(w_grid * np.conjugate(e0) * 
                    (dFRs[:,:,:,k] * sv[l] * dRdq[i,k,l])))
                else:
                    a[k,l] = a[l,k]
        df_val[i+3] = np.sum(a) # df_val[3] to df_val[5]
    end_dfr = timer()
    print('     time for dfr: ', end_dfr-start_dfr)
    wfsc = np.real(w_grid * np.conjugate(e0) * e1) # THIS WORKS
    start_ddfr = timer()
    for i in range(3):
        for j in range(3):
            if i == 0 or (i > 0 and j >= i):
                b[:,:] = 0.0
                n = -1
                for k in range(3):
                    for l in range(3): 
                        if k == 0 or (k > 0 and l >= k):
                            n = n + 1
                            b[k,l] = -(tp2/vol) * xyz_sum[n] * np.sum(wfsc * sv[k] * sv[l] *
                                dRdq[i,k,l] * dRdq[j,k,l])
                        else:
                            b[k,l] = b[l,k]
                ddf_val[i+3,j+3] = np.sum(b) # ddf_val[3] to [5] in i and j indices
            else:
                ddf_val[i+3,j+3] = ddf_val[j+3,i+3]    
    end_ddfr = timer()
    print('     time for ddfr: ', start_ddfr-end_ddfr)
    end = timer()
    print(' time for derivative calculation: ', end-start)
    if np.linalg.cond(ddf_val) < 1/sys.float_info.epsilon:
        ddf_val_inv = np.linalg.pinv(ddf_val)
    else:
        print('Derivative matrix is non-invertible! Stopping now...')
        exit()
    step = ddf_val_inv.dot(-df_val)
    return step,df_val,e1


"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
from emda.config import *

def Bf_linefit(res_arr,
               binSgnlVar,
               low_res_cutoff,
               high_res_cutoff,
               scale=1):
    from scipy import stats
    import numpy as np
    import numpy.ma as ma
    prediSigVar = np.zeros(len(res_arr), dtype='float')
    dist_low = np.sqrt((res_arr - low_res_cutoff)**2)
    lb = np.argmin(dist_low) + 1
    dist_high = np.sqrt((res_arr - high_res_cutoff)**2)
    ub = np.argmin(dist_high) + 1
    res_arr_ma = ma.masked_equal(res_arr,0.0)
    s = 1/res_arr_ma
    x_full = (s * s) / 2.0
    x_full = x_full.filled(0.0)
    slope, intercept, rv, pv, se = stats.linregress(x_full[lb:ub],
                                                    np.log(binSgnlVar[lb:ub]))
    Bfac = slope
    #print('B factor = ', Bfac, 'Intercept = ', intercept)
    ln_s = -1.0*abs(Bfac) * x_full + intercept
    prediSigVar = scale * np.exp(ln_s)
    return Bfac,prediSigVar

def estimate_map_resol(hfmap1,hfmap2):
    from emda.iotools import read_map
    from emda import restools
    import emda.fsc
    import numpy as np
    uc,arr1,origin = read_map(hfmap1)
    uc,arr2,origin = read_map(hfmap2)
    hf1 = np.fft.fftshift(np.fft.fftn(arr1))
    hf2 = np.fft.fftshift(np.fft.fftn(arr2))
    nbin,res_arr,bin_idx = restools.get_resolution_array(uc, hf1)
    bin_fsc,_,_,_,_,_ = emda.fsc.halfmaps_fsc_variance(hf1,hf2,bin_idx,nbin)
    bin_fsc = bin_fsc[bin_fsc > 0.1]
    dist = np.sqrt((bin_fsc - 0.143)**2)
    map_resol = res_arr[np.argmin(dist)]
    return map_resol

def get_map_power(mapin):
    from emda.iotools import read_map
    from emda import restools
    import fcodes_fast
    import numpy as np
    uc,arr,_ = read_map(mapin)
    hf = np.fft.fftshift(np.fft.fftn(arr))
    nx, ny, nz = hf.shape
    nbin,res_arr,bin_idx = restools.get_resolution_array(uc, hf)
    power_spectrum = fcodes_fast.calc_power_spectrum(hf,bin_idx,nbin,debug_mode,nx,ny,nz)
    print('Resolution   bin     Power')
    for i in range(len(res_arr)):
        print("{:.2f} {:.4f}".format(res_arr[i], power_spectrum[i]))
    return res_arr,power_spectrum

def get_biso_from_model(mmcif_file):
    from emda.iotools import read_mmcif
    _,_,_,_,Biso_np = read_mmcif(mmcif_file)
    Biso = np.median(Biso_np)
    return Biso

def get_biso_from_map(halfmap1,halfmap2):
    from emda.plotter import plot_nlines_log
    import numpy as np
    map_resol,sig_var,res_arr = estimate_map_resol(halfmap1,halfmap2)
    plot_nlines_log(res_arr,[sig_var],["Signal Variance"])
    low_res_cutoff = np.float32(input('Enter low resolution cutoff: '))
    high_res_cutoff = np.float32(input('Enter high resolution cutoff: '))
    Biso,preditcted_signal = Bf_linefit(res_arr,sig_var,low_res_cutoff,high_res_cutoff)
    plot_nlines_log(res_arr,[sig_var,preditcted_signal],["Signal Variance","Predicted SV"],'Predicted.eps')
    return Biso

def apply_bfactor_to_map(mapname,bf_arr,mapout):
    from emda import iotools
    import fcodes_fast
    import numpy as np
    uc,ar1,origin = iotools.read_map(mapname)
    hf1 = np.fft.fftshift(np.fft.fftn(ar1))
    nx, ny, nz = ar1.shape
    nbf = len(bf_arr)
    all_mapout = fcodes_fast.apply_bfactor_to_map(hf1,bf_arr,uc,debug_mode,nx,ny,nz,nbf)
    if mapout:
        for i in range(all_mapout.shape[3]):
            if bf_arr[i] < 0.0:
                Bcode = '_blur'+str(abs(bf_arr[i]))
            elif bf_arr[i] > 0.0:
                Bcode = '_sharp'+str(abs(bf_arr[i]))
            elif bf_arr[i] == 0.0:
                Bcode = '_unsharpened'
            filename_mrc = mapname[:-4]+Bcode+'.mrc'
            data2write = np.real(np.fft.ifftn(np.fft.ifftshift(all_mapout[:,:,:,i])))
            iotools.write_mrc(data2write,filename_mrc,uc,origin)
            print('writing done')
    return all_mapout

def map2mtz(mapname,mtzname='map2mtz.mtz',factor=1.0):
    from emda.iotools import read_map,write_3d2mtz
    import numpy as np
    uc,ar1,_ = read_map(mapname)
    uc[0] = uc[0] * factor
    uc[1] = uc[1] * factor
    uc[2] = uc[2] * factor
    hf1 = np.fft.fftshift(np.fft.fftn(ar1))
    write_3d2mtz(uc,hf1,outfile=mtzname)

def mtz2map(mtzname,map_size):
    import numpy as np
    from numpy.fft import ifftn,ifftshift
    from emda.iotools import read_mtz
    import fcodes_fast
    _,dataframe = read_mtz(mtzname)
    h = dataframe['H'].astype('int')
    k = dataframe['K'].astype('int')
    l = dataframe['L'].astype('int')
    f = dataframe['Fout0'] * np.exp(np.pi * 1j * dataframe['Pout0']/180.0)
    nx, ny, nz = map_size
    f3d = fcodes_fast.mtz2_3d(h,k,l,f,nx,ny,nz,len(f))
    data2write = np.real((ifftn(ifftshift(f3d))))
    return data2write







"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
from timeit import default_timer as timer
import sys
from emda.iotools import read_map,write_mrc
from emda.restools import get_resArr,remove_edge,create_soft_edged_kernel_pxl
import argparse
from emda.config import *

cmdl_parser = argparse.ArgumentParser(
                                      description='Computes Fourier space correlation\n')
cmdl_parser.add_argument('-h1', '--half1_map', required=True, help='Input filename for hfmap1')
cmdl_parser.add_argument('-h2', '--half2_map', required=True, help='Input filename for hfmap2')
cmdl_parser.add_argument('-k', '--kernel_size', type=np.int32, required=False, default=5, help='Kernel size (Pixels)')
cmdl_parser.add_argument('-v', '--verbose', default=False,
                         help='Verbose output')

#debug_mode = 1 # 0: no debug info, 1: debug

def newvariance(hfmap,kern_3d):
    import scipy.signal
    convmap  = scipy.signal.fftconvolve(hfmap, kern_3d, "same")
    return convmap

def get_3dnoise_variance(hf1,hf2,kern_sphere):#,uc,origin):
    noisevar = (hf1 - hf2) / 2
    loc3_A = newvariance(noisevar, kern_sphere)
    loc3_AA = newvariance(noisevar * np.conjugate(noisevar), kern_sphere)
    noise3d = (loc3_AA - loc3_A * np.conjugate(loc3_A))
    return noise3d

def get_3dtotal_variance(hf1,hf2,kern_sphere):
    fullmap = (hf1 + hf2) / 2
    loc3_A = newvariance(fullmap, kern_sphere)
    loc3_AA = newvariance(fullmap * np.conjugate(fullmap), kern_sphere)
    totalvar_3d = loc3_AA - loc3_A * np.conjugate(loc3_A)
    return totalvar_3d

def get_3d_covariance(map_list,kern_sphere):
    # function for EMDA 3D map calculation
    loc3_A = newvariance(map_list[0], kern_sphere)
    loc3_B = newvariance(map_list[1], kern_sphere)
    loc3_AB = newvariance(map_list[0] * np.conjugate(map_list[1]), kern_sphere)
    cov3_AB = loc3_AB - loc3_A * loc3_B.conj()
    return cov3_AB

def get_3d_fouriercorrelation(hf1,hf2,kern_sphere):
    # halfmaps correlation in fourier space
    import numpy as np
    from emda.iotools import mask_by_value
    loc3_A = newvariance(hf1, kern_sphere)
    loc3_B = newvariance(hf2, kern_sphere)
    loc3_AA = newvariance(hf1 * np.conjugate(hf1), kern_sphere)
    loc3_BB = newvariance(hf2 * np.conjugate(hf2), kern_sphere)
    loc3_AB = newvariance(hf1 * np.conjugate(hf2), kern_sphere)
    cov3_AB = loc3_AB - loc3_A * loc3_B.conj()
    var3_A = loc3_AA - loc3_A * loc3_A.conj()
    var3_B = loc3_BB - loc3_B * loc3_B.conj()
    #
    '''var3_A = np.where(var3_A <= 0., 0.1, var3_A)
    var3_B = np.where(var3_B <= 0., 0.1, var3_B)
    f_halfmaps_corr = mask_by_value(cov3_AB / np.sqrt(var3_A * var3_B))
    f_halfmaps_corr = cov3_AB / np.sqrt(var3_A * var3_B)
    f_halfmaps_corr = np.where(cov3_AB <= 0., 0.,f_halfmaps_corr)'''
    #
    f_halfmaps_corr = mask_by_value(cov3_AB / np.sqrt(var3_A * var3_B))
    f_fullmap_corr = 2 * f_halfmaps_corr / (1.0 + f_halfmaps_corr)
    #f_truemap_corr = np.sqrt(f_fullmap_corr)
    return f_halfmaps_corr, f_fullmap_corr, np.sqrt(f_fullmap_corr)


def fcc(half1_map, half2_map, kernel_size):
    from emda.iotools import read_map,write_mrc
    from emda.restools import get_resArr,remove_edge,create_soft_edged_kernel_pxl
    uc,half1,origin = read_map(half1_map)
    uc,half2,origin = read_map(half2_map)    
    nx,ny,nz = half1.shape
    fResArr = get_resArr(uc,nx)
    cut_mask = remove_edge(fResArr,fResArr[-1])
    cc_mask = np.zeros(shape=(nx,ny,nz),dtype='int')
    cx, cy, cz = cut_mask.shape
    dx = (nx - cx)//2
    dy = (ny - cy)//2
    dz = (nz - cz)//2
    cc_mask[dx:dx+cx, dy:dy+cy, dz:dz+cz] = cut_mask
    # Creating soft-edged mask
    kern_sphere_soft = create_soft_edged_kernel_pxl(kernel_size) 
    write_mrc(kern_sphere_soft,'kern_sphere_soft.mrc',uc,origin)
    f_halfmaps,f_fullmap,f_truemap = get_3d_fouriercorrelation(
                                        np.fft.fftshift(np.fft.fftn(half1)),
                                        np.fft.fftshift(np.fft.fftn(half2)),
                                        kern_sphere_soft)
    write_mrc(np.real(f_halfmaps) * cc_mask,'fouriercorr3d_halfmaps.mrc',uc,origin)
    write_mrc(np.real(f_fullmap) * cc_mask,'fouriercorr3d_fullmap.mrc',uc,origin)
    write_mrc(np.real(f_truemap) * cc_mask,'fouriercorr3d_truemap.mrc',uc,origin)


def main():
    args = cmdl_parser.parse_args()
    #
    uc,half1,origin = read_map(args.half1_map)
    uc,half2,origin = read_map(args.half2_map)
    #
    nx,ny,nz = half1.shape
    fResArr = get_resArr(uc,nx)
    cut_mask = remove_edge(fResArr,fResArr[-1])
    cc_mask = np.zeros(shape=(nx,ny,nz),dtype='int')
    cx, cy, cz = cut_mask.shape
    dx = (nx - cx)//2
    dy = (ny - cy)//2
    dz = (nz - cz)//2
    cc_mask[dx:dx+cx, dy:dy+cy, dz:dz+cz] = cut_mask
    
    # Creating soft-edged mask
    kern_sphere_soft = create_soft_edged_kernel_pxl(args.kernel_size) # sphere with radius of n pixles
    write_mrc(kern_sphere_soft,'kern_sphere_soft.mrc',uc,origin)

    f_halfmaps,f_fullmap,f_truemap = get_3d_fouriercorrelation(np.fft.fftshift(np.fft.fftn(half1)),
                              np.fft.fftshift(np.fft.fftn(half2)),
                              kern_sphere_soft)
    #write_mrc(np.real(f_halfmaps) * cc_mask,'fouriercorr3d_halfmaps.mrc',uc,origin)
    #write_mrc(np.real(f_fullmap) * cc_mask,'fouriercorr3d_fullmap.mrc',uc,origin)
    #write_mrc(np.real(f_truemap) * cc_mask,'fouriercorr3d_truemap.mrc',uc,origin)
    write_mrc(np.real(f_halfmaps),'fouriercorr3d_halfmaps.mrc',uc,origin)
    write_mrc(np.real(f_fullmap) ,'fouriercorr3d_fullmap.mrc',uc,origin)
    write_mrc(np.real(f_truemap) ,'fouriercorr3d_truemap.mrc',uc,origin)

if(__name__ == "__main__"):
    main()



"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
import argparse
from emda.iotools import read_map
from emda.plotter import plot_nlines
import emda.fsc
import fcodes_fast
from emda.restools import get_resolution_array
from emda.config import *

# Calculate FSC_true by removing mask effects using phase randomization 
# Date: 2019.07.08
# Modi: 2019.08.13
# Author: Rangana Warshamanage

cmdl_parser = argparse.ArgumentParser(
description='Computes fsc_true in resolutuon bins using phase randomisation\n')
cmdl_parser.add_argument('-h1', '--half1_map', required=True, help='Input filename for hfmap1')
cmdl_parser.add_argument('-h2', '--half2_map', required=True, help='Input filename for hfmap2')
cmdl_parser.add_argument('-m', '--mask_map', required=True, help='Input filename for mask')
cmdl_parser.add_argument('-r', '--res_rand', type=np.float32, required=False, help='Resol.(A) for phase randomization')
cmdl_parser.add_argument('-v', '--verbose', default=False,
                         help='Verbose output')

#debug_mode = 1

def randomize_phases(sf):
    import numpy as np
    random_phases = np.random.uniform(-np.pi, np.pi, sf.shape)
    amplitudes = np.sqrt(np.real(sf * sf.conj()))
    sf_phase_randomized = (amplitudes * 
                          np.cos(random_phases) + 
                          1j * amplitudes * 
                          np.sin(random_phases))
    return sf_phase_randomized

def get_randomized_sf(resol_grid,rho,resol_randomize=10.0):
    import fcodes_fast
    print('SF phase randomization')
    f_ori = np.fft.fftshift(np.fft.fftn(rho))
    f_all_random = randomize_phases(f_ori)
    nx,ny,nz = f_ori.shape
    f_beyond_random = fcodes_fast.add_random_phase_beyond(f_ori,f_all_random,
                                                          resol_grid,
                                                          resol_randomize,
                                                          nx,ny,nz)
    return f_beyond_random

def main():
    args = cmdl_parser.parse_args()
    # Read half maps
    uc,half1,origin = read_map(args.half1_map)
    uc,half2,origin = read_map(args.half2_map)
    uc,mask,origin  = read_map(args.mask_map)
    nbin,res_arr,bin_idx = get_resolution_array(uc,half1)
    # calculate fsc
    fsc_list = []
    # unmasked fsc
    umbin_fsc,_,_,_,_,_ = fsc.halfmaps_fsc_variance(np.fft.fftshift(np.fft.fftn(half1)),
                                                    np.fft.fftshift(np.fft.fftn(half2)),
                                                    bin_idx,
                                                    nbin)
    full_fsc_unmasked = 2.0 * umbin_fsc / (1.0 + umbin_fsc)
    fsc_list.append(full_fsc_unmasked)
    # masked fsc
    bin_fsc,_,_,_,_,_ = fsc.halfmaps_fsc_variance(np.fft.fftshift(np.fft.fftn(half1*mask)),
                                                  np.fft.fftshift(np.fft.fftn(half2*mask)),
                                                  bin_idx,
                                                  nbin)
    full_fsc_t = 2.0 * bin_fsc / (1.0 + bin_fsc)
    fsc_list.append(full_fsc_t)
    
    if args.res_rand is None: 
        # resol_rand is taken as the point where half-fsc_masked falls below 0.8 - RELION
        xx = (bin_fsc - 0.8)**2
        idx = xx.argmin()
        resol_rand = res_arr[idx]
    else:
        resol_rand = args.res_rand
        idx = np.argmin((res_arr - resol_rand)**2)

    # randomize phases 
    hf1_randomized = get_randomized_sf(uc,half1,resol_rand)
    hf2_randomized = get_randomized_sf(uc,half2,resol_rand)
    # get phase randomized maps
    randhalf1 = np.real(np.fft.ifftn(np.fft.ifftshift(hf1_randomized)))
    randhalf2 = np.real(np.fft.ifftn(np.fft.ifftshift(hf2_randomized)))
    rbin_fsc,_,_,_,_,_ = fsc.halfmaps_fsc_variance(np.fft.fftshift(np.fft.fftn(randhalf1*mask)),
                                                   np.fft.fftshift(np.fft.fftn(randhalf2*mask)),
                                                   bin_idx,
                                                   nbin)
    full_fsc_n = 2.0 * rbin_fsc / (1.0 + rbin_fsc)
    fsc_list.append(full_fsc_n)
    # fsc_true from Richard's formula
    fsc_true = (full_fsc_t - full_fsc_n) / (1 - full_fsc_n)
    fsc_true[:idx+2] = full_fsc_t[:idx+2] # replace fsc_true with fsc_masked_full upto resol_rand_idx + 2 (RELION uses 2)
    fsc_list.append(fsc_true)
    # writing fsc_true into a file
    f = open("fsc_true.txt", "w")
    f.write("Resol  full_fsc_t   full_fsc_true\n")
    for i in range(len(res_arr)):
        f.write("{} {} {}\n".format(res_arr[i],full_fsc_t[i],fsc_true[i]))
    f.close()
    plot_nlines(res_arr,fsc_list,'fsc.eps',["unmasked","fsc_t","fsc_n","fsc_true"])

if(__name__ == "__main__"):
    main()



# this code generates the mask if not supplied. 
# then ally that mask to arr.
from __future__ import absolute_import, division, print_function, unicode_literals

from emda.config import *

class MaskedMaps:
    def __init__(self,hfmap_list=None):
        self.hfmap_list = hfmap_list

    def read_halfmaps(self):
        from emda import iotools
        for n in range(0,len(self.hfmap_list),2):
            uc,arr1,origin = iotools.read_map(self.hfmap_list[n])
            uc,arr2,origin = iotools.read_map(self.hfmap_list[n+1])
        self.uc = uc
        self.arr1 = arr1
        self.arr2 = arr2
        self.origin = origin

    def generate_mask(self, arr1, arr2, smax=10):
        from emda import realsp_corr_3d
        from emda.restools import create_soft_edged_kernel_pxl
        kern = create_soft_edged_kernel_pxl(smax) # sphere with radius of n pixles
        _,fullcc3d = realsp_corr_3d.get_3d_realspcorrelation(arr1,arr2,kern)
        #iotools.write_mrc(fullcc3d,'fullcc3d.mrc',self.uc,self.origin)
        cc_mask, threshold = self.histogram(fullcc3d)
        print('threshold: ',threshold)
        mask = fullcc3d * (fullcc3d >= threshold)
        #iotools.write_mrc(mask*cc_mask,'mask.mrc',self.uc,self.origin)
        self.mask = mask*cc_mask
        #self.fullcc3d = fullcc3d
        self.arr1 = arr1 * self.mask
        self.arr2 = arr2 * self.mask        

    def create_edgemask(self, radius):
        import numpy as np
        # Remove everything outside radius
        box_radius = radius + 1
        box_size = radius * 2 + 1
        # Creating a sphere mask
        center = [box_radius, box_radius, box_radius]
        print('boxsize: ',box_size,'boxradius: ',box_radius,'center:',center)
        radius = box_radius
        X, Y, Z = np.ogrid[:box_size, :box_size, :box_size]
        dist_from_center = np.sqrt((X - center[0])**2 + (Y-center[1])**2 + (Z-center[2])**2)
        mask = dist_from_center <= radius
        return mask

    def histogram(self, arr1):
        import numpy as np
        from scipy import stats
        nx, ny, nz = arr1.shape
        maxbin = np.amax(np.array([nx//2,ny//2,nz//2]))
        counts = stats.binned_statistic(arr1.flatten(),
                                        arr1.flatten(),
                                        statistic='count',
                                        bins=maxbin,
                                        range=(0,1))[0]
        sc = counts - np.roll(counts,1)
        ulim = len(sc) - 11 + np.argmax(sc[-10:])
        cc_arr = np.linspace(0,1,maxbin)
        xc = cc_arr[:ulim] * counts[:ulim]
        xc_sum = np.sum(xc)
        isum = 0.0
        cc_arr = cc_arr[:ulim]
        for i in range(len(xc)):
            isum = isum + xc[i]
            if isum >= xc_sum/2:
                threshold = cc_arr[i]
                break
        edge_mask = self.create_edgemask(ulim)
        cc_mask = np.zeros(shape=(nx,ny,nz),dtype='bool')
        cx, cy, cz = edge_mask.shape
        dx = (nx - cx)//2
        dy = (ny - cy)//2
        dz = (nz - cz)//2
        print(dx,dy,dz)
        cc_mask[dx:dx+cx, dy:dy+cy, dz:dz+cz] = edge_mask
        return cc_mask,threshold

    def get_radial_sum(self, arr1):
        # this function not used
        import numpy as np
        from emda import iotools, restools
        import fcodes_fast
        from matplotlib import pyplot as plt
        nx,ny,nz = arr1.shape
        nbin,res_arr,bin_idx = restools.get_resolution_array(self.uc,arr1)
        sum_lst = []
        ibin_lst = []
        isum = 0.0
        isum_old = 0.0
        for ibin in range(nbin):
            ibin_sum = np.sum(arr1 * (bin_idx==ibin))
            ibin_lst.append(ibin_sum)
            isum_old = isum
            isum = isum + ibin_sum
            #if ibin_sum < 0.0 and ibin > 10: 
            if isum <= isum_old and ibin > 10:
                break
            print(ibin,ibin_sum)
            sum_lst.append(isum)
        edge_mask = self.create_edgemask(ibin)
        cc_mask = np.zeros(shape=(nx,ny,nz),dtype='bool')
        cx, cy, cz = edge_mask.shape
        dx = (nx - cx)//2
        dy = (ny - cy)//2
        dz = (nz - cz)//2
        print(dx,dy,dz)
        cc_mask[dx:dx+cx, dy:dy+cy, dz:dz+cz] = edge_mask  
        iotools.write_mrc(arr1*cc_mask,'maskedcc_map.mrc',self.uc,self.origin)    
        #plt.plot(sum_lst,"r")
        plt.plot(ibin_lst,"r")
        plt.show()



if(__name__ == "__main__"):    
    maplist = [
            '/Users/ranganaw/MRC/REFMAC/Bianka/EMD-4572/other/run_half1_class001_unfil.mrc',
            '/Users/ranganaw/MRC/REFMAC/Bianka/EMD-4572/other/run_half2_class001_unfil.mrc'
            ]       
    obj = MaskedMaps(maplist)
    obj.read_halfmaps()
    obj.generate_mask(obj.arr1, obj.arr2)


from emda.config import *

# calculate scale between two maps in resolution bins
def scale_onemap2another(f1, f2, bin_idx, res_arr):
    import numpy as np
    import fcodes_fast
    assert f1.shape == f2.shape == bin_idx.shape
    nx, ny, nz = bin_idx.shape
    scale_np = np.zeros(len(res_arr), dtype='float')
    s_grid = fcodes_fast.read_into_grid(bin_idx,
                                1.0/res_arr,
                                len(res_arr),
                                nx,ny,nz)
    print('resolution       scale')
    for i, res in enumerate(res_arr):
        slope = estimate_scale(f1[bin_idx==i], 
                                f2[bin_idx==i], 
                                s_grid[bin_idx==i])
        #scale_np[i] = params[0]
        scale_np[i] = slope
        print("{:8.4f} {:6.2f}".format(res, slope))   
    return scale_np

def estimate_scale(f1, f2, s):
    import numpy as np
    from emda.mapfit import curve_fit_3
    from scipy import stats
    x0 = np.array([1., 10.])
    if f1.ndim > 1:
        f1 = f1.flatten()
    if f2.ndim > 1:
        f1 = f2.flatten()
    if s.ndim > 1:
        f1 = s.flatten()            
    s = (s**2)/4
    #params = curve_fit_3.lsq(f1, f2, s, x0)
    # just scaling
    slope, intercept,_,_,_ = stats.linregress(np.real(f1*f1.conj()), 
                                              np.real(f2*f2.conj()))
    return slope

def scale_twomaps_by_power(f1, f2, bin_idx=None, uc=None, res_arr=None):
    from emda import restools
    import fcodes_fast
    import numpy as np
    assert f1.shape == f2.shape == bin_idx.shape
    nx, ny, nz = f1.shape
    if bin_idx is None:
        nbin,res_arr,bin_idx = restools.get_resolution_array(uc,f1)
    else:
        nbin = np.max(bin_idx) + 1
    power_1 = fcodes_fast.calc_power_spectrum(f1,bin_idx,nbin,debug_mode,nx,ny,nz)
    power_2 = fcodes_fast.calc_power_spectrum(f2,bin_idx,nbin,debug_mode,nx,ny,nz)
    scale_np = power_2/power_1
    for i, res in enumerate(res_arr):
        print("{:8.4f} {:6.2f}".format(res, scale_np[i]))
    return scale_np

def transfer_power(bin_idx,res_arr,scale):
    import fcodes_fast
    nx, ny, nz = bin_idx.shape
    scale_grid = fcodes_fast.read_into_grid(bin_idx,
                                1.0/scale,
                                len(res_arr),
                                nx,ny,nz)
    return scale_grid

if (__name__ == "__main__"):
    from emda.iotools import read_map
    import numpy as np
    from emda.restools import get_resolution_array
    maplist = ['/Users/ranganaw/MRC/REFMAC/Bianka/emda_overlay_test/emd_4572_rot180_axr010.mrc',
                #'/Users/ranganaw/MRC/REFMAC/Bianka/emda_overlay_test/emd_4572_rot180_axr010.mrc']
                #'/Users/ranganaw/MRC/REFMAC/Bianka/emda_overlay_test/emd_4572_rot180_axr010_tra200.mrc']
            '/Users/ranganaw/MRC/REFMAC/Bianka/emda_overlay_test/emd_4572_rot180_axr010_rotated20_axr100.mrc']
        
    fhf_lst = []
    for imap in maplist:
        uc,arr,_ = read_map(imap)
        fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr))))
    nbin,res_arr,bin_idx = get_resolution_array(uc,fhf_lst[0])
    '''scale_np = scale_onemap2another(fhf_lst[0],
                         fhf_lst[1],
                         bin_idx,
                         res_arr)'''
    scale_np = scale_twomaps_by_power(fhf_lst[0],
                                      fhf_lst[1],
                                      bin_idx=bin_idx,
                                      res_arr=res_arr)



# Caller script
from __future__ import absolute_import, division, print_function, unicode_literals
from emda.config import *

def read_map(mapname):
    import emda.iotools
    uc,arr,origin = emda.iotools.read_map(mapname)
    return uc,arr,origin

def read_mtz(mtzfile):
    import emda.iotools
    uc,df = emda.iotools.read_mtz(mtzfile)
    return uc,df

def write_mrc(mapdata,filename,unit_cell,map_origin=[0.0,0.0,0.0]):
    import emda.iotools
    emda.iotools.write_mrc(mapdata,
                           filename,
                           unit_cell,
                           map_origin)

def write_mtz(uc,arr,outfile='output.mtz'):
    import emda.iotools
    emda.iotools.write_3d2mtz(uc,
                              arr,
                              outfile='output.mtz')

def resample_data(target_pix_size,target_dim,uc,arr):
    import emda.iotools
    new_arr = emda.iotools.resample2staticmap(target_pix_size,
                                         target_dim,
                                         uc,
                                         arr)
    return new_arr

def estimate_map_resol(hfmap1name,hfmap2name):
    import emda.maptools
    map_resol = emda.maptools.estimate_map_resol(hfmap1name,hfmap2name)
    return map_resol

def get_map_power(mapname):
    import emda.maptools
    res_arr,power_spectrum = emda.maptools.get_map_power(mapname)
    return res_arr,power_spectrum

def get_biso_from_model(mmcif_file):
    import emda.maptools
    biso = emda.maptools.get_biso_from_model(mmcif_file)
    return biso

def get_biso_from_map(halfmap1,halfmap2):
    import emda.maptools
    biso = emda.maptools.get_biso_from_map(halfmap1,halfmap2)
    return biso

def apply_bfactor_to_map(mapname,bf_arr,mapout):
    import emda.maptools
    all_mapout = emda.maptools.apply_bfactor_to_map(mapname,
                                                    bf_arr,
                                                    mapout)
    return all_mapout
    
def map2mtz(mapname,mtzname):
    import emda.maptools
    emda.maptools.map2mtz(mapname,mtzname=mtzname)

def mtz2map(mtzname,map_size):
    import emda.maptools
    data2write = emda.maptools.mtz2map(mtzname,map_size)
    return data2write

def lowpass_map(mapname, resol):
    import emda.lowpass_map
    emda.lowpass_map.lowpass_map(mapname, resol)

def half2full(half1name,half2name,outfile):
    import emda.half2full
    uc,arr1,origin = read_map(half1name)
    uc,arr2,origin = read_map(half2name)
    fullmap = emda.half2full.half2full(arr1, arr2)
    write_mrc(fullmap,outfile,uc,origin)
    return fullmap

def map_transform(mapname,t,r,ax,outname):
    import emda.transform_map
    transformed_map = emda.transform_map.map_transform(mapname,
                                                       t,
                                                       r,
                                                       tuple(ax),
                                                       outname)
    return transformed_map

def halfmap_fsc(half1name,half2name,filename=None):
    import emda.restools
    import emda.fsc
    import fcodes_fast
    import numpy as np
    import os
    uc,arr1,origin = read_map(half1name)
    uc,arr2,origin = read_map(half2name)
    hf1 = np.fft.fftshift(np.fft.fftn(arr1))
    hf2 = np.fft.fftshift(np.fft.fftn(arr2))
    nbin,res_arr,bin_idx = emda.restools.get_resolution_array(uc,hf1)
    #bin_fsc,noisevar,signalvar,totalvar,_,_ = emda.fsc.halfmaps_fsc_variance(hf1,
    #                                                        hf2,bin_idx,nbin)
    _,_,noisevar,signalvar,totalvar,bin_fsc,bincount = fcodes_fast.calc_fsc_using_halfmaps(hf1,
                                                    hf2,
                                                    bin_idx,
                                                    nbin,
                                                    debug_mode,
                                                    hf1.shape[0],
                                                    hf1.shape[1],
                                                    hf1.shape[2])
    if filename is not None: 
        tdata = open(filename, "w") 
        tdata.write('halfmap1 file: %s\n' %os.path.abspath(half1name))
        tdata.write('halfmap2 file: %s\n' %os.path.abspath(half2name))
        tdata.write('\n')
        tdata.write('bin # \n')  
        tdata.write('resolution (Ang.) \n') 
        tdata.write('signal variance \n')  
        tdata.write('noise variance \n')  
        tdata.write('total variance \n')  
        tdata.write('halfmap fsc \n')   
        tdata.write('# reflx \n')                                                
        for i in range(len(res_arr)):
            sv = signalvar[i]
            nv = noisevar[i]
            tv = totalvar[i]
            fsc = bin_fsc[i]
            nfc = bincount[i]
            tdata.write("{:-3d} {:-6.2f} {:-14.4f} {:-14.4f} {:-14.4f} {:-14.4f} {:-10d}\n".format(i, 
                                                                      res_arr[i], 
                                                                      sv, nv, tv, 
                                                                      fsc,nfc))
    print('Bin    Resolution     FSC')
    for i in range(len(res_arr)):
        print("{:5d} {:6.2f} {:14.4f}".format(i, res_arr[i], bin_fsc[i]))
    return res_arr,bin_fsc

def twomap_fsc(map1name,map2name,fobj=None):
    import emda.restools
    import emda.fsc
    import numpy as np
    import os
    uc,arr1,origin = read_map(map1name)
    uc,arr2,origin = read_map(map2name)
    f1 = np.fft.fftshift(np.fft.fftn(arr1))
    f2 = np.fft.fftshift(np.fft.fftn(arr2))
    nbin,res_arr,bin_idx = emda.restools.get_resolution_array(uc,f1)
    bin_fsc,f1f2_covar = emda.fsc.anytwomaps_fsc_covariance(f1,f2,bin_idx,nbin)
    if fobj is not None: 
        fobj.write('map1 file: %s\n' %os.path.abspath(map1name))
        fobj.write('map2 file: %s\n' %os.path.abspath(map2name))
        fobj.write('\n')
        fobj.write('bin # \n')  
        fobj.write('resolution (Ang.) \n')  
        fobj.write('fsc \n')                                                   
        for bin, fsc in enumerate(bin_fsc):
            fobj.write("{:5d} {:6.2f} {:6.3f}\n".format(bin, 
                                                         res_arr[bin], 
                                                         fsc))
    print('Bin      Resolution     FSC')
    for bin, fsc in enumerate(bin_fsc):
        print("{:5d} {:6.2f} {:6.3f}".format(bin, res_arr[bin], fsc))
    return res_arr,bin_fsc

def singlemap_fsc(map1name):
    import emda.restools
    import emda.fsc
    import numpy as np
    from emda.fakehalf import fakehalf
    uc,arr1,origin = read_map(map1name)
    f1 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr1)))
    f2 = fakehalf(f1)
    data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(f2))))
    write_mrc(data2write,'fakehalf.mrc',uc,origin)
    nbin,res_arr,bin_idx = emda.restools.get_resolution_array(uc,f1)
    #bin_fsc,f1f2_covar = emda.fsc.anytwomaps_fsc_covariance(f1,f2,bin_idx,nbin)
    bin_fsc,noisevar,signalvar,totalvar,fo,eo = emda.fsc.halfmaps_fsc_variance(f1,
                                                            f2,bin_idx,nbin)
    print('Resolution bin     FSC')
    for i in range(len(res_arr)):
        print("{:.2f} {:.4f}".format(res_arr[i], bin_fsc[i]))
    return res_arr,bin_fsc

def mask_from_halfmaps(half1name, half2name, radius, maskname='halfmap_mask.mrc',norm=False):
    import emda.maskmap_class
    from emda.realsp_corr_3d import normalized
    uc,arr1,origin = read_map(half1name)
    uc,arr2,origin = read_map(half2name)
    # normalized maps
    if norm:
        arr1 = normalized(map=arr1, uc=uc)
        arr2 = normalized(map=arr2, uc=uc)
    obj_maskmap = emda.maskmap_class.MaskedMaps()
    obj_maskmap.generate_mask(arr1, arr2, smax=radius)
    mask = obj_maskmap.mask
    write_mrc(mask,maskname,uc,origin)
    return mask

def sphere_kernel_softedge(radius=5):
    import emda.restools
    kernel = emda.restools.create_soft_edged_kernel_pxl(radius)
    return kernel

def overlay_maps(maplist,masklist,tra,rot,ax,ncy,res,fobj,interp):
    from emda.mapfit import mapoverlay
    theta_init = [tuple(ax),rot]
    mapoverlay.main(maplist=maplist,
                    masklist=masklist,
                    ncycles=ncy,
                    t_init=tra,
                    theta_init=theta_init,
                    smax=res,
                    fobj=fobj,
                    interp=interp)

def average_maps(maplist,masklist,tra,rot,ax,ncy,res,fobj,interp):
    from emda.mapfit import mapaverage
    theta_init = [tuple(ax),rot]
    mapaverage.main(maplist=maplist,
                    masklist=masklist,
                    ncycles=ncy,
                    t_init=tra,
                    theta_init=theta_init,
                    smax=res,
                    fobj=fobj,
                    interp=interp)

def realsp_correlation(half1map, half2map, kernel_size, model=None, model_resol=None, mask_map=None):
    from emda.realsp_corr_3d import rcc
    rcc(half1map, half2map, kernel_size, model, model_resol, mask_map)

def fouriersp_correlation(half1_map, half2_map, kernel_size):
    from emda.fouriersp_corr_3d import fcc
    fcc(half1_map, half2_map, kernel_size)

def map_model_validate(half1map, half2map, modelfpdb, model1pdb, mask, mapsize, modelresol):
    from emda.map_fsc import map_model_fsc
    map_model_fsc(half1map, half2map, modelfpdb, model1pdb, mask, mapsize, modelresol)

def difference_map(mapname1, mapname2):
    import numpy as np
    from emda.mapfit import difference
    uc,arr1,origin = read_map(mapname1)
    _,arr2,_ = read_map(mapname2)
    f1 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr1)))
    f2 = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr2)))
    difference.difference_map(f1, f2, uc,origin)
    '''dm1_dm2 = np.real(np.fft.ifftshift(
                         np.fft.ifftn(
                         np.fft.ifftshift(f1 - f2))))
    dm2_dm1 = np.real(np.fft.ifftshift(
                         np.fft.ifftn(
                         np.fft.ifftshift(f2 - f1))))
    write_mrc(dm1_dm2,'diffmap_m1-m2.mrc',uc,origin)
    write_mrc(dm2_dm1,'diffmap_m2-m1.mrc',uc,origin)'''

def applymask(mapname, maskname, outname):
    uc,arr1,origin = read_map(mapname)
    _,mask,_ = read_map(maskname)
    write_mrc(arr1*mask,outname,uc,origin)

def scale_map2map(staticmap,map2scale,outfile):
    import numpy as np
    from emda.scale_maps import scale_twomaps_by_power,transfer_power
    from emda import restools
    uc,arr1,origin = read_map(staticmap)
    uc,arr2,origin = read_map(map2scale)
    f1 = np.fft.fftshift(np.fft.fftn(arr1))
    f2 = np.fft.fftshift(np.fft.fftn(arr2))
    nbin,res_arr,bin_idx = restools.get_resolution_array(uc,f1)
    scale_grid = transfer_power(bin_idx,res_arr,
                                scale_twomaps_by_power(f1,
                                                       f2,
                                                       bin_idx=bin_idx,
                                                       res_arr=res_arr))
    data2write = np.real(np.fft.ifftn(np.fft.ifftshift(
                                                       f2 * scale_grid)))
    write_mrc(data2write,outfile,uc,origin)
    return data2write

def bestmap(hf1name, hf2name, outfile, mode=1, knl=5):
    import numpy as np
    from emda import restools
    from emda.mapfit import bestmap
    uc,arr1,origin = read_map(hf1name)
    uc,arr2,origin = read_map(hf2name)
    f1 = np.fft.fftshift(np.fft.fftn(arr1))
    f2 = np.fft.fftshift(np.fft.fftn(arr2))
    nbin,res_arr,bin_idx = restools.get_resolution_array(uc,f1)
    f_map = bestmap.bestmap(f1, f2, bin_idx, nbin,mode=mode, kernel_size=knl)
    data2write = np.real(np.fft.ifftn(np.fft.ifftshift(f_map)))
    write_mrc(data2write,outfile,uc,origin)

def predict_fsc(hf1name, hf2name, nparticles, mask=None):
    import numpy as np
    from emda import restools,fsc,plotter
    uc,arr1,_ = read_map(hf1name)
    uc,arr2,_ = read_map(hf2name)
    if mask is not None:
        _,msk,_ = read_map(mask)
    else:
        msk = 1.0
    f1 = np.fft.fftshift(np.fft.fftn(arr1 * msk))
    f2 = np.fft.fftshift(np.fft.fftn(arr2 * msk))
    nbin,res_arr,bin_idx = restools.get_resolution_array(uc,f1)
    fsc_lst = fsc.predict_fsc(f1,f2,bin_idx,nbin,np.asarray(nparticles, dtype='float'))
    print('Resolution     FSC')
    #for i in range(len(res_arr)):
        #print("{:.2f} {:.4f}".format(res_arr[i], fsc[i]))
    #    print(res_arr[i], fsc[i,:])
    labels = [str(i) for i in nparticles]
    labels.append('reference')
    plotter.plot_nlines(res_arr, fsc_lst, 
                    curve_label=labels, mapname='fsc_predicted.eps')
    return fsc_lst,res_arr,bin_idx,nbin

def prepare_refmac_data(hf1name,hf2name,outfile,bfac=None,maskname=None):
    import numpy as np
    from emda import iotools,restools
    import fcodes_fast
    import os
    uc,arr1,origin = read_map(hf1name)
    uc,arr2,origin = read_map(hf2name)
    if maskname is None:
        msk = 1.0
    if maskname is not None:
        _,msk,_ = read_map(maskname)
        if arr1.shape != msk.shape:
            print('mask dim and map dim do not match!')
            print('map dim:', arr1.shape, 'mask dim:', msk.shape)
            exit()
    if bfac is None:
        bfac = np.array([0.0], dtype='float')
    else:
        bfac = np.asarray(bfac, dtype='float')
        bfac = np.insert(bfac,0,0.0)
    hf1 = np.fft.fftshift(np.fft.fftn(arr1 * msk))
    hf2 = np.fft.fftshift(np.fft.fftn(arr2 * msk))
    # stats from half maps
    nx,ny,nz = hf1.shape
    maxbin = np.amax(np.array([nx//2,ny//2,nz//2]))
    nbin,res_arr,bin_idx,sgrid = fcodes_fast.resolution_grid(uc,debug_mode,maxbin,nx,ny,nz)
    res_arr = res_arr[:nbin]
    _,_,nvar,svar,tvar,binfsc,bincount = fcodes_fast.calc_fsc_using_halfmaps(hf1,
                                                    hf2,
                                                    bin_idx,
                                                    nbin,
                                                    debug_mode,
                                                    hf1.shape[0],
                                                    hf1.shape[1],
                                                    hf1.shape[2])

    tdata = open('table_variances.txt', "w") 
    tdata.write('halfmap1 file: %s\n' %os.path.abspath(hf1name))
    tdata.write('halfmap2 file: %s\n' %os.path.abspath(hf2name))
    tdata.write('\n')
    tdata.write('bin # \n')  
    tdata.write('resolution (Ang.) \n') 
    tdata.write('signal variance \n')  
    tdata.write('noise variance \n')  
    tdata.write('total variance \n')  
    tdata.write('halfmap fsc \n')   
    tdata.write('# reflx \n')                                                
    for i in range(len(res_arr)):
        sv = svar[i]
        nv = nvar[i]
        tv = tvar[i]
        fsc = binfsc[i]
        nfc = bincount[i]
        tdata.write("{:-3d} {:-6.2f} {:-14.4f} {:-14.4f} {:-14.4f} {:-14.4f} {:-10d}\n".format(i, 
                                                                  res_arr[i], 
                                                                  sv, nv, tv, 
                                                                  fsc,nfc))
    print('Bin    Resolution     FSC')
    for i in range(len(res_arr)):
        print("{:5d} {:6.2f} {:14.4f}".format(i, res_arr[i], binfsc[i]))
    # output mtz file    
    iotools.write_3d2mtz_refmac(uc,
                                sgrid,
                                (hf1+hf2)/2.0,
                                (hf1-hf2),
                                bfac,
                                outfile=outfile)
    
def overall_cc(map1name, map2name, space='real', maskname=None):
    from emda import cc
    import numpy as np
    uc, arr1, origin = read_map(map1name)
    uc, arr2, origin = read_map(map2name)
    if maskname is not None:
        uc, msk, origin = read_map(maskname)
        arr1 = arr1 * msk
        arr2 = arr2 * msk
    if space == 'fourier':
        print('Overall CC calculation in Fourier space')
        f1 = np.fft.fftn(arr1)
        f2 = np.fft.fftn(arr2)
        occ = cc.cc_overall_fouriersp(f1=f1, f2=f2)
        print('Overall Correlation in Fourier space= ', occ)
    else:
        print('Overall CC calculation in Real/Image space')
        occ = cc.cc_overall_realsp(map1=arr1,map2=arr2)
        print('Overall Correlation in real space= ', occ)
    return occ

def mirror_map(mapname):
    # gives the inverted copy of the map
    import numpy as np
    uc, arr, origin = read_map(mapname)
    data = np.real(np.fft.ifftn(np.conjugate(np.fft.fftn(arr))))
    write_mrc(data,'mirror.mrc',uc,origin)

def model2map(modelxyz,dim,resol,cell,bfac=0.0):
    import numpy as np
    from emda.iotools import run_refmac_sfcalc
    from emda.maptools import mtz2map
    import gemmi as gm
    # check for valid sampling: 
    if np.any(np.mod(dim, 2)) != 0:
        dim = dim + 1
    # check for minimum sampling
    min_pix_size = resol / 2 # in Angstrom
    min_dim = np.asarray(cell[:3], dtype='float') / min_pix_size
    min_dim = np.ceil(min_dim).astype(int)
    if np.any(np.mod(min_dim, 2)) != 0:
        min_dim = min_dim + 1
    if min_dim[0] > dim[0]:
        print('Minimum dim should be: ', min_dim)
        exit()
    # replace/add cell and write model.cif
    a, b, c = cell[:3]
    structure = gm.read_structure(modelxyz)
    structure.cell.set(a, b, c, 90.0, 90.0, 90.0)
    structure.make_mmcif_document().write_file('model.cif')
    # run refmac using model.cif just created
    run_refmac_sfcalc('./model.cif',resol,bfac)
    modelmap = mtz2map('sfcalc_from_crd.mtz',dim)
    return modelmap

def read_atomsf(atom, fpath=None):
    from emda import iotools
    import subprocess
    if fpath is None:
        CMD = 'echo $CLIBD'
        p = subprocess.Popen(CMD, stdout=subprocess.PIPE, shell=True, \
            executable='/bin/bash')
        list_of_strings = [x.decode('utf-8').rstrip('\n') \
            for x in iter(p.stdout.readlines())]
        fpath = list_of_strings[0]+'/atomsf.lib'
    a, b = iotools.read_atomsf(atom, fpath=fpath)
    return a, b




# fitting function

def fsc_between_static_and_transfomed_map(staticmap,movingmap,bin_idx,rm,t,cell,nbin):
    import fcodes_fast
    from emda.mapfit import utils
    from emda import fsc
    nx, ny, nz = staticmap.shape
    st,_,_,_ = fcodes_fast.get_st(nx,ny,nz,t)
    frt_full = utils.get_FRS(rm,movingmap * st, interp='cubic')[:,:,:,0]
    f1f2_fsc,_ = fsc.anytwomaps_fsc_covariance(staticmap,
                                               frt_full,
                                               bin_idx,
                                               nbin)
    return f1f2_fsc

def get_ibin(bin_fsc):
    import numpy as np
    bin_fsc = bin_fsc[bin_fsc > 0.1]
    dist = np.sqrt((bin_fsc - 0.4)**2)
    ibin = np.argmin(dist) + 1
    if ibin % 2 != 0: ibin = ibin - 1
    ibin = min([len(dist), ibin])
    return ibin

def run_fit(emmap1,smax,rotmat,t,slf,ncycles,ifit,fobj,interp,dfs_full=None):
    import numpy as np
    from emda.mapfit import emfit_Class
    from emda.mapfit.frequency_marching import frequency_marching
    from emda.mapfit import interp_derivatives
    fsc_lst = []
    for i in range(5):
        if i==0:
            smax = smax # A
            if emmap1.res_arr[0] < smax: 
                ibin = 2
                print('Fitting starts at ',emmap1.res_arr[ibin],' (A) instead!' )
            else:
                dist = np.sqrt((emmap1.res_arr - smax)**2)
                ibin = np.argmin(dist) + 1 
                if ibin % 2 != 0: ibin = ibin - 1
                ibin = min([len(dist), ibin])
                print('Fitting starts at ',emmap1.res_arr[ibin],' (A)' )
            ibin_old = ibin
            f1f2_fsc = fsc_between_static_and_transfomed_map(
                                        staticmap=emmap1.fo_lst[0],
                                        movingmap=emmap1.fo_lst[ifit],
                                        bin_idx=emmap1.bin_idx,
                                        rm=rotmat,
                                        t=t,
                                        cell=emmap1.map_unit_cell,
                                        nbin=emmap1.nbin)
            fsc_lst.append(f1f2_fsc)
        else:
            # Apply initial rotation and translation to calculate fsc
            f1f2_fsc = fsc_between_static_and_transfomed_map(
                                        emmap1.fo_lst[0],
                                        emmap1.fo_lst[ifit],
                                        emmap1.bin_idx,
                                        rotmat,
                                        t,
                                        emmap1.map_unit_cell,
                                        emmap1.nbin)
            ibin = get_ibin(f1f2_fsc)
            if ibin_old == ibin: 
                fsc_lst.append(f1f2_fsc)
                fobj.write('\n')
                fobj.write('FSC between static and moving maps\n')
                fobj.write('\n')
                fobj.write('bin#\n')
                fobj.write('resolution (A)\n')
                fobj.write('start FSC\n')
                fobj.write('end FSC\n')
                fobj.write('\n')
                for j in range(len(emmap1.res_arr)):
                    print(emmap1.res_arr[j], fsc_lst[0][j], fsc_lst[1][j])
                    fobj.write("{:5d} {:6.2f} {:8.4f} {:8.4f}\n".format(
                        j,
                        emmap1.res_arr[j],
                        fsc_lst[0][j],
                        fsc_lst[1][j]))
                break
            else: 
                ibin_old = ibin
        if ibin == 0:
            print('Cannot find a solution! Stopping now...')
            exit()
        static_cutmap,cBIdx, cbin = frequency_marching(emmap1.eo_lst[0],
                                        emmap1.bin_idx,
                                        emmap1.res_arr,
                                        bmax=ibin)  
        moving_cutmap,_, _ = frequency_marching(emmap1.eo_lst[ifit],
                                            emmap1.bin_idx,
                                            emmap1.res_arr,
                                            bmax=ibin)
        moving_cutmap_f1,_, _ = frequency_marching(emmap1.fo_lst[ifit],
                                            emmap1.bin_idx,
                                            emmap1.res_arr,
                                            bmax=ibin)
        if dfs_full is not None:
            # cut dfs_full for current size
            dfs = interp_derivatives.cut_dfs4interp(dfs_full,cbin)
            #
        else: 
            dfs = None
        assert static_cutmap.shape == moving_cutmap.shape
        emmap1.cfo_lst = [moving_cutmap_f1]
        emmap1.ceo_lst = [static_cutmap,moving_cutmap]
        emmap1.cbin_idx = cBIdx
        emmap1.cdim = moving_cutmap.shape
        emmap1.cbin = cbin
        fit = emfit_Class.EmFit(emmap1,interp=interp,dfs=dfs)
        if ibin < slf or slf == 0: slf = ibin
        slf = min([ibin, slf])
        fit.minimizer(ncycles, t, rotmat, smax_lf=slf, fobj=fobj)
        ncycles = ncycles # tweaking this you can change later # cycles
        t = fit.t_accum
        rotmat = fit.rotmat
    return rotmat, t